@@ -1,7 +1,24 @@
-// Generally useful JavaScript used by TWiki
+/*
+This collection of javascript functions is deprecated.
+Use the new TWiki library classes instead:
+twikilib.js
+twikiArray.js
+twikiCSS.js
+twikiEvent.js
+twikiForm.js
+twikiFunction.js
+twikiHTML.js
+twikiPref.js
+twikiString.js, twikiStringUnicodeChars.js
+twikiWindow.js
+
+When converting to the new classes: some functions may have changed name or parameters.
+
+*/
 
 var POPUP_WINDOW_WIDTH = 500;
 var POPUP_WINDOW_HEIGHT = 480;
+var POPUP_ATTRIBUTES = "titlebar=0,resizable,scrollbars";
 
 var TWIKI_PREF_COOKIE_NAME = "TWIKIPREF";
 var COOKIE_PREF_SEPARATOR = "|"; // separates key-value pairs
@@ -31,7 +48,8 @@ var LOWER_ALPHA_CHARS		= "a-z";
 var NUMERIC_CHARS			= "\\d";
 var MIXED_ALPHA_CHARS		= UPPER_ALPHA_CHARS + LOWER_ALPHA_CHARS;
 var MIXED_ALPHANUM_CHARS	= MIXED_ALPHA_CHARS + NUMERIC_CHARS;
-var WIKIWORD_REGEX = "^" + "[" + UPPER_ALPHA_CHARS + "]" + "+" + "[" + LOWER_ALPHA_CHARS + "]" + "+" + "[" + UPPER_ALPHA_CHARS + "]" + "+" + "[" + MIXED_ALPHANUM_CHARS + "]" + "*";
+var LOWER_ALPHANUM_CHARS	= LOWER_ALPHA_CHARS + NUMERIC_CHARS;
+var WIKIWORD_REGEX = "^" + "[" + UPPER_ALPHA_CHARS + "]" + "+" + "[" + LOWER_ALPHANUM_CHARS + "]" + "+" + "[" + UPPER_ALPHA_CHARS + "]" + "+" + "[" + MIXED_ALPHANUM_CHARS + "]" + "*";
 var ALLOWED_URL_CHARS = MIXED_ALPHANUM_CHARS + "-_^";
 
 // TWiki namespace
@@ -64,15 +82,30 @@ function initForm() {
 
 // Launch a fixed-size help window
 function launchTheWindow(inPath, inWeb, inTopic, inSkin, inTemplate) {
-	var template = "";
-	var skin = "";
+
+	var pathComps = [];
+	if (inWeb != undefined) pathComps.push(inWeb);
+	if (inTopic != undefined) pathComps.push(inTopic);
+	var pathString = inPath + pathComps.join("/");
+	
+	var params = [];
 	if (inSkin != undefined && inSkin.length > 0) {
-		skin = "?skin=" + inSkin;
+		params.push("skin=" + inSkin);
 	}
 	if (inTemplate != undefined && inTemplate.length > 0) {
-		template = "?template=" + inTemplate;
+		params.push("template=" + inTemplate);
 	}
-	var win = open(inPath + inWeb + "/" + inTopic + skin + template, inTopic, "titlebar=0,width=" + POPUP_WINDOW_WIDTH + ",height=" + POPUP_WINDOW_HEIGHT + ",resizable,scrollbars");
+	var paramsString = params.join(";");
+	if (paramsString.length > 0) paramsString = "?" + paramsString;
+	var name = (inTopic != undefined) ? inTopic : "";
+	
+	var attributes = [];
+	attributes.push("width=" + POPUP_WINDOW_WIDTH);
+	attributes.push("height=" + POPUP_WINDOW_HEIGHT);
+	attributes.push(POPUP_ATTRIBUTES);
+	var attributesString = attributes.join(",");
+	
+	var win = open(pathString + paramsString, name, attributesString);
 	if (win) win.focus();
 	return false;
 }
@@ -131,14 +164,17 @@ function setClassList(element, classlist) {
 
 // Determine the first index of a string in an array.
 // Return -1 if the string is not found.
-function indexOf(element, array) {
-	if (!array || array.length == undefined) return -1;
-	var i = array.length;
-	while (i--)	{
-		if (array[i] == element) return i;
+// WATCH OUT: the refactored function in twiki.Array returns null with an
+// invalid array, but CSS class manipulation functions still rely on a 
+// return value of -1
+function indexOf(inElement, inArray) {
+		if (!inArray || inArray.length == undefined) return -1;
+		var i, ilen = inArray.length;
+		for (i=0; i<ilen; ++i) {
+			if (inArray[i] == inElement) return i;
+		}
+		return -1;
 	}
-	return -1;
-}
 
 // Applies the given function to all elements in the document of
 // the given tag type
@@ -226,7 +262,7 @@ function removePunctuation(inValue) {
 /**
 Combines removePunctuation and removeSpaces.
 */
-function removeSpacesAndPunctiation(inValue) {
+function removeSpacesAndPunctuation(inValue) {
 	return removePunctuation(removeSpaces(inValue));
 }
 
@@ -327,9 +363,8 @@ Writes a TWiki preference value. If the TWiki preference of given name already e
 Characters '|' and '=' are reserved as separators.
 @param inPrefName (String): name of the preference to write, for instance 'SHOWATTACHMENTS'
 @param inPrefValue (String): value to write, for instance '1'
-@param inExpiryDate (Number): (optional) expiry date of cookie; if not set uses COOKIE_EXPIRY_TIME
 */
-function setPref(inPrefName, inPrefValue, inExpiryDate) {
+function setPref(inPrefName, inPrefValue) {
 	var prefName = _getSafeString(inPrefName);
 	var prefValue = (isNaN(inPrefValue)) ? _getSafeString(inPrefValue) : inPrefValue;
 	var cookieString = _getPrefCookie();
@@ -342,7 +377,7 @@ function setPref(inPrefName, inPrefValue, inExpiryDate) {
 	// else not found, so don't remove an existing entry
 	var keyvalueString = prefName + COOKIE_PREF_VALUE_SEPARATOR + prefValue;
 	prefs.push(keyvalueString);
-	_writePrefValues(prefs, inExpiryDate);
+	_writePrefValues(prefs);
 }
 
 /**
@@ -384,6 +419,7 @@ Finds a key-value pair in an array.
 @return The first occurrence of a key-value pair, where key == inKey; null if none is found.
 */
 function _getKeyValue (inKeyValues, inKey) {
+	if (!inKeyValues) return null;
 	var i = inKeyValues.length;
 	while (i--) {
 		var keyvalue = inKeyValues[i].split(COOKIE_PREF_VALUE_SEPARATOR);
@@ -399,6 +435,7 @@ Finds the location of a key-value pair in an array.
 @return The location of the first occurrence of a key-value tuple, where key == inKey; -1 if none is found.
 */
 function _getKeyValueLoc (inKeyValues, inKey) {
+	if (!inKeyValues) return null;
 	var i = inKeyValues.length;
 	while (i--) {
 		var keyvalue = inKeyValues[i].split(COOKIE_PREF_VALUE_SEPARATOR);
@@ -410,14 +447,13 @@ function _getKeyValueLoc (inKeyValues, inKey) {
 /**
 Writes a cookie with the stringified array values of inValues.
 @param inValues: (Array) an array with key-value tuples
-@param inExpiryDate: (Number) (optional) expiry date of cookie; if not set uses COOKIE_EXPIRY_TIME
 */
-function _writePrefValues (inValues, inExpiryDate) {
+function _writePrefValues (inValues) {
 	var cookieString = (inValues != null) ? inValues.join(COOKIE_PREF_SEPARATOR) : '';
-	var expdate = (inExpiryDate != null) ? new Date(inExpiryDate) : new Date ();
-	FixCookieDate (expdate); // Correct for Mac date bug - call only once for given Date object!
-	expdate.setTime (expdate.getTime() + COOKIE_EXPIRY_TIME);
-	SetCookie(TWIKI_PREF_COOKIE_NAME, cookieString, expdate);
+	var expiryDate = new Date ();
+	FixCookieDate (expiryDate); // Correct for Mac date bug - call only once for given Date object!
+	expiryDate.setTime (expiryDate.getTime() + COOKIE_EXPIRY_TIME);
+	SetCookie(TWIKI_PREF_COOKIE_NAME, cookieString, expiryDate, '/');
 }
 
 /**
@@ -427,9 +463,7 @@ Gets the TWiki pref cookie; creates a new cookie if it does not exist.
 function _getPrefCookie () {
 	var cookieString = GetCookie(TWIKI_PREF_COOKIE_NAME);
 	if (cookieString == undefined) {
-		// Cookie does not exist yet. Create a new one by writing without values.
-		_writePrefValues();
-		cookieString = GetCookie(TWIKI_PREF_COOKIE_NAME);
+		cookieString = "";
 	}
 	return cookieString;
 }
@@ -478,18 +512,19 @@ function FixCookieDate (date) {
 //      the cookie does not exist.
 //
 function GetCookie (name) {
-  var arg = name + "=";
-  var alen = arg.length;
-  var clen = document.cookie.length;
-  var i = 0;
-  while (i < clen) {
-    var j = i + alen;
-    if (document.cookie.substring(i, j) == arg)
-      return getCookieVal (j);
-    i = document.cookie.indexOf(" ", i) + 1;
-    if (i == 0) break; 
-  }
-  return null;
+	var arg = name + "=";
+	var alen = arg.length;
+	var clen = document.cookie.length;
+	var i = 0;
+	while (i < clen) {
+		var j = i + alen;
+		if (document.cookie.substring(i, j) == arg) {
+			return getCookieVal(j);
+		}
+		i = document.cookie.indexOf(" ", i) + 1;
+		if (i == 0) break; 
+	}
+	return null;
 }
 //
 //  Function to create or update a cookie.
@@ -520,11 +555,12 @@ function GetCookie (name) {
 //      SetCookie (myCookieVar, cookieValueVar, null, "/myPath", null, true);
 //
 function SetCookie (name,value,expires,path,domain,secure) {
-  document.cookie = name + "=" + escape (value) +
+  var cookieString = name + "=" + escape (value) +
     ((expires) ? "; expires=" + expires.toGMTString() : "") +
     ((path) ? "; path=" + path : "") +
     ((domain) ? "; domain=" + domain : "") +
     ((secure) ? "; secure" : "");
+    document.cookie = cookieString;
 }
 
 //  Function to delete a cookie. (Sets expiration date to start of epoch)
@@ -537,10 +573,7 @@ function SetCookie (name,value,expires,path,domain,secure) {
 //             no domain was specified when creating the cookie.
 //
 function DeleteCookie (name,path,domain) {
-  if (GetCookie(name)) {
-    document.cookie = name + "=" +
-      ((path) ? "; path=" + path : "") +
-      ((domain) ? "; domain=" + domain : "") +
-      "; expires=Thu, 01-Jan-70 00:00:01 GMT";
-  }
+	if (GetCookie(name)) {
+		document.cookie = name + "=" + ((path) ? "; path=" + path : "") + ((domain) ? "; domain=" + domain : "") + "; expires=Thu, 01-Jan-70 00:00:01 GMT";
+	}
 }
