@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 2005-2006 TWiki Contributors.
+# Copyright (C) 2005-2007 TWiki Contributors.
 # All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -33,7 +33,7 @@ for all login managers.
 
 On it's own, an object of this class is used when you specify 'none' in
 the security setup section of
-[[%SCRIPTURL%/configure%SCRIPTSUFFIX%][configure]]. When it is used,
+[[%SCRIPTURL{"configure"}%][configure]]. When it is used,
 logins are not supported. If you want to authenticate users then you should
 consider TemplateLogin or ApacheLogin, which are subclasses of this class.
 
@@ -46,7 +46,25 @@ $TWiki::cfg{Trace}{Client.pm}. The tracing is done in such a way as to
 let the perl optimiser optimise out the trace function as a no-op if tracing
 is disabled.
 
+Here's an overview of how it works:
+
+Early in TWiki::new, the login manager is created. The creation of the login manager does two things:
+   1 If sessions are in use, it loads CGI::Session but doesn't initialise the session yet.
+   1 Creates the login manager object
+Slightly later in TWiki::new, loginManager->loadSession is called.
+   1 Calls loginManager->getUser to get the username *before* the session is created
+      * TWiki::Client::ApacheLogin looks at REMOTE_USER
+      * TWiki::Client::TemplateLogin just returns undef
+   1 reads the TWIKISID cookie to get the SID (or the TWIKISID parameters in the CGI query if cookies aren't available, or IP2SID mapping if that's enabled).
+   1 Creates the CGI::Session object, and the session is thereby read.
+   1 If the username still isn't known, reads it from the cookie. Thus TWiki::Client::ApacheLogin overrides the cookie using REMOTE_USER, and TWiki::Client::TemplateLogin *always* uses the session.
+
+Later again in TWiki::new, plugins are given a chance to *override* the username found from the loginManager.
+
+The last step in TWiki::new is to find the user, using whatever user mapping manager is in place.
+
 ---++ ObjectData =twiki=
+
 The TWiki object this login manager is attached to.
 
 =cut
@@ -76,18 +94,19 @@ $M3 = chr(7);
 
 =pod
 
----++ StaticMethod makeClient( $twiki ) -> $TWiki::Client
+---++ StaticMethod makeLoginManager( $twiki ) -> $TWiki::Client
+
 Factory method, used to generate a new TWiki::Client object
 for the given session.
 
 =cut
 
-sub makeClient {
+sub makeLoginManager {
     my $twiki = shift;
     ASSERT($twiki->isa( 'TWiki')) if DEBUG;
 
-    if( $TWiki::cfg{UseClientSessions} && 
-	!$twiki->inContext( 'command_line' )) {
+    if( $TWiki::cfg{UseClientSessions} &&
+          !$twiki->inContext( 'command_line' )) {
 
         my $use = 'use CGI::Session';
         if( $TWiki::cfg{Sessions}{UseIPMatching} ) {
@@ -162,7 +181,7 @@ sub _IP2SID {
     return undef unless $ip; # no IP address, can't map
 
     my %ips;
-    if( open( IPMAP, '<', $TWiki::cfg{Sessions}{Dir}.'/ip2sid' )) {
+    if( open( IPMAP, '<', $TWiki::cfg{TempfileDir}.'/ip2sid' )) {
         local $/ = undef;
         %ips = map { split( /:/, $_ ) } split( /\r?\n/, <IPMAP> );
         close(IPMAP);
@@ -170,7 +189,7 @@ sub _IP2SID {
     if( $sid ) {
         # known SID, map the IP addr to it
         $ips{$ip} = $sid;
-        open( IPMAP, '>', $TWiki::cfg{Sessions}{Dir}.'/ip2sid') ||
+        open( IPMAP, '>', $TWiki::cfg{TempfileDir}.'/ip2sid') ||
           die "Failed to open ip2sid map for write. Ask your administrator to make sure that the {Sessions}{Dir} is writable by the webserver user.";
         print IPMAP map { "$_:$ips{$_}\n" } keys %ips;
         close(IPMAP);
@@ -183,21 +202,31 @@ sub _IP2SID {
 
 =pod
 
----++ ObjectMethod loadSession()
+---++ ObjectMethod loadSession($defaultUser) -> $login
 
 Get the client session data, using the cookie and/or the request URL.
 Set up appropriate session variables in the twiki object and return
 the login name.
 
+$defaultUser is a username to use if one is not available from other
+sources. The username passed when you create a TWiki instance is
+passed in here.
+
 =cut
 
 sub loadSession {
-    return undef unless( $TWiki::cfg{UseClientSessions} );
-
-    my $this = shift;
+    my ($this, $defaultUser) = @_;
     my $twiki = $this->{twiki};
 
-    return undef if $twiki->inContext( 'command_line' );
+    # Try and get the user from the webserver
+    my $authUser = $this->getUser( $this ) || $defaultUser;
+
+    unless( $TWiki::cfg{UseClientSessions} ) {
+        $this->userLoggedIn( $authUser ) if $authUser;
+        return $authUser;
+    }
+
+    return $authUser if $twiki->inContext( 'command_line' );
 
     my $query = $twiki->{cgiQuery};
 
@@ -209,9 +238,17 @@ sub loadSession {
     } else {
         _trace($this, "No cookie ");
     }
-
-    # Try and get the user from the webserver
-    my $authUser = $this->getUser( $this );
+    
+    # Item3568: CGI::Session from 4.0 already does the -d and creates the
+    # sessions directory if it does not exist. For performance reasons we
+    # only test for and create session file directory for older CGI::Session
+    if( $CGI::Session::VERSION < 4.0 ) {
+        unless ( -d $TWiki::cfg{TempfileDir} ) {
+            unless ( mkdir($TWiki::cfg{TempfileDir}) ) {
+                die "Could not create $TWiki::cfg{TempfileDir} for session files";
+            }
+        }
+    }
 
     # First, see if there is a cookied session, creating a new session
     # if necessary.
@@ -220,18 +257,18 @@ sub loadSession {
         my $sid = _IP2SID();
         if( $sid ) {
             $this->{_cgisession} = CGI::Session->new(
-                undef, $sid, { Directory => $TWiki::cfg{Sessions}{Dir} } );
+                undef, $sid, { Directory => $TWiki::cfg{TempfileDir} } );
         } else {
             $this->{_cgisession} = CGI::Session->new(
                 undef, undef,
-                { Directory => $TWiki::cfg{Sessions}{Dir} } );
+                { Directory => $TWiki::cfg{TempfileDir} } );
             _trace($this, "New IP2SID session");
             _IP2SID( $this->{_cgisession}->id() );
         }
     } else {
         $this->{_cgisession} = CGI::Session->new(
             undef, $query,
-            { Directory => $TWiki::cfg{Sessions}{Dir} } );
+            { Directory => $TWiki::cfg{TempfileDir} } );
     }
 
     die CGI::Session->errstr() unless $this->{_cgisession};
@@ -251,10 +288,14 @@ sub loadSession {
     if( $authUser ) {
         _trace($this, "Session says user is $authUser");
     } else {
-        $authUser = $twiki->{remoteUser};
-        _trace($this, "TWiki object says user is $authUser");
+        # Use remote user provided from "new TWiki" call. This is mainly
+        # for testing.
+        $authUser = $defaultUser;
+        _trace($this, "TWiki object says user is $authUser") if $authUser;
     }
 
+    $authUser ||= $defaultUser;
+
     # is this a logout?
     if( $query && $query->param( 'logout' ) ) {
         _trace($this, "User is logging out");
@@ -274,6 +315,7 @@ sub loadSession {
 =pod
 
 ---++ ObjectMethod checkAccess()
+
 Check if the script being run in this session is authorised for execution.
 If not, throw an access control exception.
 
@@ -306,6 +348,7 @@ sub checkAccess {
 =pod
 
 ---++ ObjectMethod finish
+
 Complete processing after the client's HTTP request has been responded
 to. Flush the user's session (if any) to disk.
 
@@ -329,7 +372,8 @@ sub finish {
 =pod
 
 ---++ StaticMethod expireDeadSessions()
-Delete sessions that are sitting around but are really expired.
+
+Delete sessions and passthrough files that are sitting around but are really expired.
 This *assumes* that the sessions are stored as files.
 
 This is a static method, but requires TWiki::cfg. It is designed to be
@@ -338,42 +382,27 @@ run from a session or from a cron job.
 =cut
 
 sub expireDeadSessions {
-	my $time = time() || 0;
+    my $time = time() || 0;
     my $exp = $TWiki::cfg{Sessions}{ExpireAfter} || 36000; # 10 hours
     $exp = -$exp if $exp < 0;
 
-	opendir(D, $TWiki::cfg{Sessions}{Dir}) || return;
-	foreach my $file ( grep { /cgisess_[0-9a-f]{32}/ } readdir(D) ) {
+    opendir(D, $TWiki::cfg{TempfileDir}) || return;
+    foreach my $file ( grep { /^(passthru|cgisess)_[0-9a-f]{32}/ } readdir(D) ) {
         $file = TWiki::Sandbox::untaintUnchecked(
-            $TWiki::cfg{Sessions}{Dir}.'/'.$file );
-		my @stat = stat( $file );
-        # Kill small old files. They can't be valid sessions.
-		# Ignore tiny new files. They can't be complete sessions.
-        if( defined($stat[7]) && $stat[7] <= 50 ) {
-            my $lat = $stat[8] || $stat[9] || $stat[10] || 0;
-            unlink $file if( $time - $lat >= $exp );
-            next;
-		}
-
-		open(F, $file) || next;
-		my $session = <F>;
-		close F;
-
-        # SMELL: security hazard?
-        $session = TWiki::Sandbox::untaintUnchecked( $session );
-
-        my $D;
-		eval $session;
-		next if ( $@ );
-        # The session is expired if it is empty, hasn't been accessed in ages
-        # or has exceeded its registered expiry time.
-        if( !$D || $time >= $D->{_SESSION_ATIME} + $exp ||
-              $D->{_SESSION_ETIME} && $time >= $D->{_SESSION_ETIME} ) {
-            unlink( $file );
-            next;
-        }
-	}
-	closedir D;
+            $TWiki::cfg{TempfileDir}.'/'.$file );
+        my @stat = stat( $file );
+        # CGI::Session updates the session file each time a browser views a
+        # topic setting the access and expiry time as values in the file. This 
+        # also sets the mtime (modification time) for the file which is all we need.
+        # We know that the expiry time is mtime + $TWiki::cfg{Sessions}{ExpireAfter}
+        # so we do not need to waste execution time opening and reading the file.
+        # We just check the mtime. mtime is confirmed set in both Windows and Linux
+        # As a fallback we also check ctime. Files are deleted when they expire.
+        my $lat = $stat[9] || $stat[10] || 0;
+        unlink $file if ( $time - $lat >= $exp );
+        next;
+    }
+    closedir D;
 }
 
 =pod
@@ -400,24 +429,27 @@ sub userLoggedIn {
             $this->{_cgisession} =
               CGI::Session->new(
                   undef, $twiki->{cgiQuery},
-                  { Directory => $TWiki::cfg{Sessions}{Dir} } );
+                  { Directory => $TWiki::cfg{TempfileDir} } );
             die CGI::Session->errstr() unless $this->{_cgisession};
         }
-
-        if( $authUser && $authUser ne $TWiki::cfg{DefaultUserLogin} ) {
-            _trace($this, "Session is authenticated");
-            $this->{_cgisession}->param( 'AUTHUSER', $authUser );
-            $twiki->enterContext( 'authenticated' );
-        } else {
-            _trace($this, "Session is NOT authenticated");
-            # if we are not authenticated, expire any existing session
-            $this->{_cgisession}->clear( [ 'AUTHUSER' ] );
-            $twiki->leaveContext( 'authenticated' );
-        }
-	# flush the session, to try to fix Item1820 and Item2234
-	$this->{_cgisession}->flush();
-	die $this->{_cgisession}->errstr() if $this->{_cgisession}->errstr();
-	_trace($this, "Flushed");
+    }
+    if( $authUser && $authUser ne $TWiki::cfg{DefaultUserLogin} ) {
+        _trace($this, "Session is authenticated");
+        $this->{_cgisession}->param( 'AUTHUSER', $authUser )
+          if( $TWiki::cfg{UseClientSessions} );
+        $twiki->enterContext( 'authenticated' );
+    } else {
+        _trace($this, "Session is NOT authenticated");
+        # if we are not authenticated, expire any existing session
+        $this->{_cgisession}->clear( [ 'AUTHUSER' ] )
+          if( $TWiki::cfg{UseClientSessions} );
+        $twiki->leaveContext( 'authenticated' );
+    }
+    if( $TWiki::cfg{UseClientSessions} ) {
+        # flush the session, to try to fix Item1820 and Item2234
+        $this->{_cgisession}->flush();
+        die $this->{_cgisession}->errstr() if $this->{_cgisession}->errstr();
+        _trace($this, "Flushed");
     }
 }
 
@@ -500,6 +532,7 @@ sub _rewriteFORM {
 =pod
 
 ---++ ObjectMethod endRenderingHandler()
+
 This handler is called by getRenderedVersion just before the plugins
 postRenderingHandler. So it is passed all HTML text just before it is
 printed.
@@ -561,6 +594,7 @@ sub addCookie {
 =pod
 
 ---++ ObjectMethod modifyHeader( \%header )
+
 Modify a HTTP header
    * =\%header= - header entries
 
@@ -584,8 +618,8 @@ sub modifyHeader {
 =pod
 
 ---++ ObjectMethod redirectCgiQuery( $url )
+
 Generate an HTTP redirect on STDOUT, if you can. Return 1 if you did.
-Don't forget to pass all query parameters through.
    * =$url= - target of the redirection.
 
 =cut
@@ -628,6 +662,7 @@ sub redirectCgiQuery {
 =pod
 
 ---++ ObjectMethod getSessionValues() -> \%values
+
 Get a name->value hash of all the defined session variables
 
 =cut
@@ -643,6 +678,7 @@ sub getSessionValues {
 =pod
 
 ---++ ObjectMethod getSessionValue( $name ) -> $value
+
 Get the value of a session variable.
 
 =cut
@@ -657,6 +693,7 @@ sub getSessionValue {
 =pod
 
 ---++ ObjectMethod setSessionValue( $name, $value )
+
 Set the value of a session variable.
 We do not allow setting of AUTHUSER.
 
@@ -677,14 +714,15 @@ sub setSessionValue {
 
 =pod
 
----++ ObjectMethod clearSessionValue( $name )
+---++ ObjectMethod clearSessionValue( $name ) -> $boolean
+
 Clear the value of a session variable.
 We do not allow setting of AUTHUSER.
 
 =cut
 
 sub clearSessionValue {
-    my( $this,, $key ) = @_;
+    my( $this, $key ) = @_;
 
     # We do not allow clearing of AUTHUSER.
     if( $this->{_cgisession} &&
@@ -751,7 +789,7 @@ sub getUser {
 sub _LOGIN {
     #my( $twiki, $params, $topic, $web ) = @_;
     my $twiki = shift;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
 
     return '' if $twiki->inContext( 'authenticated' );
@@ -766,7 +804,7 @@ sub _LOGIN {
 
 sub _LOGOUTURL {
     my( $twiki, $params, $topic, $web ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
 
     return $twiki->getScriptUrl(
@@ -778,7 +816,7 @@ sub _LOGOUTURL {
 
 sub _LOGOUT {
     my( $twiki, $params, $topic, $web ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
 
     return '' unless $twiki->inContext( 'authenticated' );
@@ -793,7 +831,7 @@ sub _LOGOUT {
 
 sub _AUTHENTICATED {
     my( $twiki, $params ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
 
     if( $twiki->inContext( 'authenticated' )) {
@@ -805,7 +843,7 @@ sub _AUTHENTICATED {
 
 sub _CANLOGIN {
     my( $twiki, $params ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
     if( $twiki->inContext( 'can_login' )) {
         return $params->{then} || 1;
@@ -816,7 +854,7 @@ sub _CANLOGIN {
 
 sub _SESSION_VARIABLE {
     my( $twiki, $params ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
     my $name = $params->{_DEFAULT};
 
@@ -833,7 +871,7 @@ sub _SESSION_VARIABLE {
 
 sub _LOGINURL {
     my( $twiki, $params ) = @_;
-    my $this = $twiki->{client};
+    my $this = $twiki->{loginManager};
     ASSERT($this->isa('TWiki::Client')) if DEBUG;
     return $this->loginUrl();
 }
