@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 2001-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 2001-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -62,17 +62,19 @@ BEGIN {
 
 =pod
 
----++ ClassMethod new ( $session, $web, $form )
+---++ ClassMethod new ( $session, $web, $form, $def )
 
    * $web - default web to recover form from, if $form doesn't specify a web
    * =$form= - topic name to read form definition from
+   * =$def= - optional. a reference to a list of field definitions. if present,
+              these definitions will be used, rather than those in =$form=.
 
 May throw TWiki::OopsException
 
 =cut
 
 sub new {
-    my( $class, $session, $web, $form, $noNameCheck ) = @_;
+    my( $class, $session, $web, $form, $def ) = @_;
     my $this = bless( {}, $class );
 
     ( $web, $form ) =
@@ -80,17 +82,26 @@ sub new {
 
     my $store = $session->{store};
 
-    # Read topic that defines the form
-    unless( $store->topicExists( $web, $form ) ) {
-        return undef;
-    }
-    my( $meta, $text ) =
-      $store->readTopic( $session->{user}, $web, $form, undef );
-
     $this->{session} = $session;
     $this->{web} = $web;
     $this->{topic} = $form;
-    $this->{fields} = $this->_parseFormDefinition( $text );
+
+    unless ( $def ) {
+
+      # Read topic that defines the form
+      unless( $store->topicExists( $web, $form ) ) {
+        return undef;
+      }
+      my( $meta, $text ) =
+	$store->readTopic( $session->{user}, $web, $form, undef );
+
+      $this->{fields} = $this->_parseFormDefinition( $text );
+
+    } else {
+
+      $this->{fields} = $def;
+
+    }
 
     # Expand out values arrays in the definition
     # SMELL: this should be done lazily
@@ -178,11 +189,11 @@ sub _parseFormDefinition {
 
             $vals ||= '';
             $vals = $this->{session}->handleCommonTags($vals,$this->{web},$this->{topic});
-
+	    $vals =~ s/<\/?(nop|noautolink)\/?>//go;
             $vals =~ s/^\s*//go;
             $vals =~ s/\s*$//go;
 
-            # SMELL: WTF is this??? This looks like a really bad hack!
+            # SMELL: What is this??? This looks like a hack!
             if( $vals eq '$users' ) {
                 $vals = $TWiki::cfg{UsersWebName} . '.' .
                   join( ", ${TWiki::cfg{UsersWebName}}.",
@@ -254,7 +265,7 @@ sub _getPossibleFieldValues {
     my @defn = ();
     my $inBlock = 0;
     foreach( split( /\r?\n/, $text ) ) {
-        if( /^\s*\|.*Name[^|]*\|/ ) {
+        if( /^\s*\|\s*\*Name\*\s*\|/ ) {
             $inBlock = 1;
         } else {
             if( /^\s*\|\s*([^|]*)\s*\|/ ) {
@@ -314,6 +325,7 @@ sub _link {
 =pod
 
 ---++ ObjectMethod renderForEdit( $web, $topic, $meta ) -> $html
+
    * =$web= the web of the topic being rendered
    * =$topic= the topic being rendered
    * =$meta= the meta data for the form
@@ -370,6 +382,7 @@ sub renderForEdit {
                 if( defined( $value )) {
                     $value = $session->handleCommonTags( $value, $web,
                                                          $topic );
+                    $value = TWiki::expandStandardEscapes( $value ); # Item2837
                 }
             }
             $value = '' unless defined $value;  # allow 0 values
@@ -391,6 +404,7 @@ sub renderForEdit {
 =pod
 
 ---++ ObjectMethod renderFieldForEdit( $fieldDef, $web, $topic, $value) -> $html
+
    * =$fieldDef= the field being rendered
    * =$web= the web of the topic being rendered
    * =$topic= the topic being rendered
@@ -401,7 +415,6 @@ from $meta. Plugins can provide a handler that extends the set of supported
 types
 
 SMELL: this should be a method on a field class
-SMELL: JSCalendarContrib ought to provide a 'date' handler.
 
 =cut
 
@@ -423,7 +436,6 @@ sub renderFieldForEdit {
     my $item;
     my %attrs;
     my @defaults;
-    my $selected;
 
     $name = $this->cgiName( $name );
 
@@ -434,8 +446,31 @@ sub renderFieldForEdit {
     if( $output ) {
         $value = $output;
 
+    } elsif( $type eq 'date' ) {
+      $size = 10 if( !$size || $size < 1 );
+      $value = CGI::textfield({ name => $name,
+				id => 'id'.$name,
+				size=> $size,
+				value => $value,
+				class=> 'twikiInputField twikiEditFormDateField'});
+      require TWiki::Contrib::JSCalendarContrib;
+      unless ( $@ ) {
+	my $ifFormat = $TWiki::cfg{JSCalendarContrib}{format} || '%e %b %Y';
+	TWiki::Contrib::JSCalendarContrib::addHEAD( 'twiki' );
+	$value .= '%TWISTY{link="" noscript="hide" start="show" prefix="&nbsp;"}%';
+	$value .= CGI::image_button( -name => 'calendar',
+				     -onclick =>
+				     "return showCalendar('id$name','$ifFormat')",
+				     -src=> $TWiki::cfg{PubUrlPath} . '/' .
+				       $TWiki::cfg{SystemWebName} .
+				       '/JSCalendarContrib/img.gif',
+				     -alt => 'Calendar',
+				     -class => 'twikiButton twikiEditFormCalendarButton' );
+	$value .= '%ENDTWISTY%';
+	$value = $session->{renderer}->getRenderedVersion( $session->handleCommonTags( $value, $web, $topic ) );
+      }
     } elsif( $type eq 'text' ) {
-        $value = CGI::textfield( -class => 'twikiEditFormTextField',
+        $value = CGI::textfield( -class => 'twikiInputField twikiEditFormTextField',
                                  -name => $name,
                                  -size => $size,
                                  -value => $value );
@@ -450,7 +485,6 @@ sub renderFieldForEdit {
         my $renderedValue = $session->{renderer}->getRenderedVersion
           ( $session->handleCommonTags( $value, $web, $topic ));
         $value = CGI::hidden( -name => $name,
-                              -class => 'twikiEditFormLabelField',
                               -value => $value );
         $value .= CGI::div( { class => 'twikiEditFormLabelField' },
                             $renderedValue );
@@ -462,26 +496,73 @@ sub renderFieldForEdit {
             $cols = $1;
             $rows = $2;
         }
-        $value = CGI::textarea( -class => 'twikiEditFormTextAreaField',
+        $value = CGI::textarea( -class => 'twikiInputField twikiEditFormTextAreaField',
                                 -cols => $cols,
                                 -rows => $rows,
                                 -name => $name,
                                 -default => "\n".$value );
 
-    } elsif( $type eq 'select' ) {
+    } elsif( $type =~ /^select/ ) {
         $options = $fieldDef->{value};
         ASSERT( ref( $options )) if DEBUG;
+        my $minSize = $size;
+        my $maxSize = $size;
+        if( $size =~ /([0-9]+)\.\.([0-9]+)/ ) {
+            ( $minSize, $maxSize ) = ( $1, $2 );
+        }
+        my $isMulti  = ( $type =~ /\+multi/ );
+        my $isValues = ( $type =~ /\+values/ );
         my $choices = '';
         foreach $item ( @$options ) {
-            $selected = ( $item eq $value );
-            $item =~ s/<nop/&lt\;nop/go;
-            if( $selected ) {
-                $choices .= CGI::option({ selected=>'selected' }, $item );
-            } else {
-                $choices .= CGI::option( $item );
+	    $item = &TWiki::urlDecode($item);
+            my $params = {
+	      class=>'twikiEditFormOption'
+	    };
+            my $itemValue = $item;
+            if( $isValues ) {
+                if( $item =~ /^(.*?[^\\])=(.*)$/ ) {
+                    $item = $1;
+                    $itemValue = $2;
+                    $params->{value} = $itemValue;
+                }
+                $item =~ s/\\=/=/g;
             }
+            if( defined $itemValue && defined $value ) {
+                my $selected;
+                if( $isMulti ) {
+                    $selected = ( $value =~ /^(.*,)?\s*$itemValue\s*(,.*)?$/ );
+                } else {
+                    $selected = ( $itemValue eq $value );
+                }
+                $params->{selected} = 'selected' if $selected;
+            }
+            $item =~ s/<nop/&lt\;nop/go;
+            $choices .= CGI::option( $params, $item );
+        }
+        $size = scalar @$options;
+        if( $size > $maxSize ) {
+            $size = $maxSize;
+        } elsif( $size < $minSize ) {
+            $size = $minSize;
+        }
+        my $params = { 
+	  class=>'twikiEditFormSelect',
+	  name=>$name, 
+	  size=>$size 
+	};
+        if( $isMulti ) {
+            $params->{'multiple'}='on';
+            $value  = CGI::Select( $params, $choices );
+            # Item2410: We need a dummy control to detect the case where
+            #           all checkboxes have been deliberately unchecked
+	    # Item3061:
+	    # Don't use CGI, it will insert the value from the query
+	    # once again and we need an empt field here.
+	    $value .= '<input type="hidden" name="'.$name.'" value="" />';
+        }
+        else {
+            $value  = CGI::Select( $params, $choices );
         }
-        $value = CGI::Select( { name=>$name, size=>$size }, $choices );
 
     } elsif( $type =~ /^checkbox/ ) {
         $options = $fieldDef->{value};
@@ -490,12 +571,12 @@ sub renderFieldForEdit {
             my $boxes = scalar( @$options );
             $extra = CGI::br();
             $extra .= CGI::button
-              ( -class => 'twikiEditFormCheckboxButton',
+              ( -class => 'twikiCheckBox twikiEditFormCheckboxButton',
                 -value => $session->{i18n}->maketext('Set all'),
                 -onClick => 'checkAll(this,2,'.$boxes.',true)' );
             $extra .= '&nbsp;';
             $extra .= CGI::button
-              ( -class => 'twikiEditFormCheckboxButton',
+              ( -class => 'twikiCheckBox twikiEditFormCheckboxButton',
                 -value => $session->{i18n}->maketext('Clear all'),
                 -onClick => 'checkAll(this,1,'.$boxes.',false)');
         }
@@ -518,14 +599,20 @@ sub renderFieldForEdit {
                                       -defaults => \@defaults,
                                       -columns => $size,
                                       -attributes => \%attrs );
+        # Item2410: We need a dummy control to detect the case where
+        #           all checkboxes have been deliberately unchecked
+	# Item3061:
+	# Don't use CGI, it will insert the value from the query
+	# once again and we need an empt field here.
+        $value .= '<input type="hidden" name="'.$name.'" value="" />';
 
     } elsif( $type eq 'radio' ) {
         $options = $fieldDef->{value};
         ASSERT( ref( $options )) if DEBUG;
-        $selected = '';
+        my $selected = '';
         foreach $item ( @$options ) {
             $attrs{$item} =
-              { class=>'twikiEditFormRadioField twikiRadioButton',
+              { class=>'twikiRadioButton twikiEditFormRadioField',
                 label=>$session->handleCommonTags( $item, $web, $topic ) };
 
             $selected = $item if( $item eq $value );
@@ -553,6 +640,7 @@ sub renderFieldForEdit {
 =pod
 
 ---++ ObjectMethod renderHidden( $meta ) -> $html
+
 Render form fields found in the meta as hidden inputs, so they pass
 through edits untouched.
 
@@ -592,6 +680,7 @@ sub renderHidden {
 =pod
 
 ---++ ObjectMethod cgiName( $field ) -> $string
+
 Generate the 'name' of the CGI parameter used to represent a field.
 
 =cut
@@ -606,14 +695,18 @@ sub cgiName {
 =pod
 
 ---++ ObjectMethod getFieldValuesFromQuery($query, $metaObject) -> ( $seen, \@missing )
+
 Extract new values for form fields from a query.
+
    * =$query= - the query
    * =$metaObject= - the meta object that is storing the form values
 
 For each field, if there is a value in the query, use it.
 Otherwise if there is already entry for the field in the meta, keep it.
 
-Returns the number of fields which had values provided by the query, and a references to an array of the names of mandatory fields that were missing from the query.
+Returns the number of fields which had values provided by the query, 
+and a references to an array of the names of mandatory fields that were 
+missing from the query.
 
 =cut
 
@@ -637,14 +730,36 @@ sub getFieldValuesFromQuery {
 
         my $value = $query->param( $param );
 
-        $seen++ if defined( $value );
+        if (defined $value) {
+            $seen++;
+            if ($this->{session}->inContext('edit')) {
+                $value  =  TWiki::expandStandardEscapes( $value );
+            }
+        }
 
         # checkbox and multi both allow multiple values
         if( $fieldDef->{type} =~ /^checkbox|\+multi/ ) {
             my @values = $query->param( $param );
-            $value = shift @values;
-            foreach my $val (@values) {
-                $value .= ", $val";
+            if ($#values>=0) {
+              if ($#values==0) {
+                @values = split /\,|%2C/, $values[0];
+              }
+              my %vset = ();
+              foreach my $val (@values) {
+                $val =~ s/^\s*//o;
+                $val =~ s/\s*$//o;
+                $vset{$val} = (defined $val && $val =~ /\S/); # skip empty values
+              }
+              $value = '';
+              my $isValues = ( $fieldDef->{type} =~ /\+values/ );
+              foreach my $flditem (@{$fieldDef->{value}}) {
+                $flditem =~ s/^.*?[^\\]=(.*)$/$1/ if $isValues; # get the real value 
+                # Maintain order of definition
+                if ($vset{$flditem}) {
+                  $value .= ', ' if $value;
+                  $value .= $flditem;
+                }
+              }
             }
         }
 
@@ -687,7 +802,9 @@ sub getFieldValuesFromQuery {
 =pod
 
 ---++ ObjectMethod isTextMergeable( $name ) -> $boolean
+
    * =$name= - name of a form field (value of the =name= attribute)
+
 Returns true if the type of the named field allows it to be text-merged.
 
 If the form does not define the field, it is assumed to be mergeable.
@@ -708,7 +825,9 @@ sub isTextMergeable {
 =pod
 
 ---++ ObjectMethod getField( $name ) -> \%row
+
    * =$name= - name of a form field (value of the =name= attribute)
+
 Returns the field, or undef if the form does not define the field.
 
 =cut
@@ -724,6 +843,7 @@ sub getField {
 =pod
 
 ---++ ObjectMethod getFields() -> \@fields
+
 Return a list containing references to field name/value pairs.
 Each entry in the list has a {name} field and a {value} field. It may
 have other fields as well, which caller should ignore. The
@@ -739,8 +859,10 @@ sub getFields {
 =pod
 
 ---++ StaticMethod renderForDisplay($templates, $meta )
+
    * =$templates= ref to templates singleton
    * =$meta= - meta object containing the form to be rendered
+
 Static because we want to be able to do this without a form definition.
 
 SMELL: Why? Is reading the form topic such a big burden?
@@ -758,7 +880,6 @@ sub renderForDisplay {
     my $name = $form->{name};
 
     my $text = $templates->expandTemplate('FORM:display:header');
-    $text =~ s/%A_TITLE%/$name/g;
 
 	my $rowTemplate = $templates->expandTemplate('FORM:display:row');
     my @fields = $meta->find( 'FIELD' );
@@ -775,6 +896,7 @@ sub renderForDisplay {
         }
     }
     $text .= $templates->expandTemplate('FORM:display:footer');
+    $text =~ s/%A_TITLE%/$name/g;
     return $text;
 }
 
