@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 2000-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 2000-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -126,6 +126,8 @@ Compose fully qualified URL
    * =$web=    - Web name, e.g. ='Main'=
    * =$topic=  - Topic name, e.g. ='WebNotify'=
    * =$script= - Script name, e.g. ='view'=
+   * =...= - an arbitrary number of name,value parameter pairs that will be url-encoded and added to the url. The special parameter name '#' is reserved for specifying an anchor. e.g. <tt>getScriptUrl('x','y','view','#'=>'XXX',a=>1,b=>2)</tt> will give <tt>.../view/x/y?a=1&b=2#XXX</tt>
+
 Return: =$url=       URL, e.g. ="http://example.com:80/cgi-bin/view.pl/Main/WebNotify"=
 
 *Since:* TWiki::Plugins::VERSION 1.000 (7 Dec 2002)
@@ -133,10 +135,12 @@ Return: =$url=       URL, e.g. ="http://example.com:80/cgi-bin/view.pl/Main/WebN
 =cut
 
 sub getScriptUrl {
-    my( $web, $topic, $script ) = @_;
+    my $web = shift;
+    my $topic = shift;
+    my $script = shift;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
 
-    return $TWiki::Plugins::SESSION->getScriptUrl( 1, $script, $web, $topic );
+    return $TWiki::Plugins::SESSION->getScriptUrl( 1, $script, $web, $topic, @_ );
 }
 
 =pod
@@ -250,7 +254,7 @@ sub getSessionValue {
 #   my( $key ) = @_;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
 
-    return $TWiki::Plugins::SESSION->{client}->getSessionValue( @_ );
+    return $TWiki::Plugins::SESSION->{loginManager}->getSessionValue( @_ );
 }
 
 
@@ -258,7 +262,7 @@ sub getSessionValue {
 
 ---+++ setSessionValue( $key, $value ) -> $boolean
 
-Set a session value via the client session module
+Set a session value.
    * =$key=   - Session key
    * =$value= - Value associated with key
 Return: true if function succeeded
@@ -271,31 +275,32 @@ sub setSessionValue {
 #   my( $key, $value ) = @_;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
 
-    $TWiki::Plugins::SESSION->{client}->setSessionValue( @_ );
+    $TWiki::Plugins::SESSION->{loginManager}->setSessionValue( @_ );
 }
 
 =pod
 
 ---+++ clearSessionValue( $key ) -> $boolean
 
-Clear a session value via the client session module
-   * =$key=   - Session key
-Return: true if function succeeded
+Clear a session value that was set using =setSessionValue=.
+   * =$key= - name of value stored in session to be cleared. Note that
+   you *cannot* clear =AUTHUSER=.
+Return: true if the session value was cleared
 
 *Since:* TWiki::Plugins::VERSION 1.1
 
 =cut
 
 sub clearSessionValue {
-#   my( $key, $value ) = @_;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
 
-    $TWiki::Plugins::SESSION->{client}->clearSessionValue( @_ );
+    return $TWiki::Plugins::SESSION->{loginManager}->clearSessionValue( @_ );
 }
 
 =pod
 
 ---+++ getContext() -> \%hash
+
 Get a hash of context identifiers representing the currently active
 context.
 
@@ -372,6 +377,9 @@ Return: =$value=  Preferences value; empty string if not set
       * WebPreferences topic has: =* Set WEBBGCOLOR = #FFFFC0=
       * =my $webColor = TWiki::Func::getPreferencesValue( 'WEBBGCOLOR', 'Sandbox' );=
 
+*NOTE:* As of TWiki4.1, if =$NO_PREFS_IN_TOPIC= is enabled in the plugin, then
+preferences set in the plugin topic will be ignored.
+
 =cut
 
 sub getPreferencesValue {
@@ -397,6 +405,9 @@ __Note__: This function will will *only* work when called from the Plugin.pm fil
 
 *Since:* TWiki::Plugins::VERSION 1.021 (27 Mar 2004)
 
+*NOTE:* As of TWiki4.1, if =$NO_PREFS_IN_TOPIC= is enabled in the plugin, then
+preferences set in the plugin topic will be ignored.
+
 =cut
 
 sub getPluginPreferencesValue {
@@ -423,6 +434,9 @@ Return: =$value=  Preferences flag ='1'= (if set), or ="0"= (for preferences val
       * Use ="MYPLUGIN_SHOWHELP"= for =$key=
       * =my $showHelp = TWiki::Func::getPreferencesFlag( "MYPLUGIN_SHOWHELP" );=
 
+*NOTE:* As of TWiki4.1, if =$NO_PREFS_IN_TOPIC= is enabled in the plugin, then
+preferences set in the plugin topic will be ignored.
+
 =cut
 
 sub getPreferencesFlag {
@@ -443,6 +457,9 @@ __Note__: This function will will *only* work when called from the Plugin.pm fil
 
 *Since:* TWiki::Plugins::VERSION 1.021 (27 Mar 2004)
 
+*NOTE:* As of TWiki4.1, if =$NO_PREFS_IN_TOPIC= is enabled in the plugin, then
+preferences set in the plugin topic will be ignored.
+
 =cut
 
 sub getPluginPreferencesFlag {
@@ -630,28 +647,49 @@ sub permissionsSet {
 
 =pod
 
----+++ checkAccessPermission( $type, $wikiName, $text, $topic, $web ) -> $boolean
+---+++ checkAccessPermission( $type, $wikiName, $text, $topic, $web, $meta ) -> $boolean
 
-Check access permission for a topic based on the [[%TWIKIWEB%.TWikiAccessControl]] rules
-   * =$type=     - Access type, e.g. ='VIEW'=, ='CHANGE'=, ='CREATE'=
-   * =$wikiName= - WikiName of remote user, e.g. ="PeterThoeny"=. If =$wikiName= is '', 0 or undef then access is always *permitted*.
-   * =$text=     - Topic text, optional. If 'perl false' (undef, 0 or ''), topic =$web.$topic= is consulted
+Check access permission for a topic based on the
+[[%TWIKIWEB%.TWikiAccessControl]] rules
+   * =$type=     - Access type, required, e.g. ='VIEW'=, ='CHANGE'=.
+   * =$wikiName= - WikiName of remote user, required, e.g. ="PeterThoeny"=.
+     If =$wikiName= is '', 0 or =undef= then access is *always permitted*.
+   * =$text=     - Topic text, optional. If 'perl false' (undef, 0 or ''),
+     topic =$web.$topic= is consulted. =$text= may optionally contain embedded
+     =%META:PREFERENCE= tags. Provide this parameter if:
+      1 You are setting different access controls in the text to those defined
+      in the stored topic,
+      1 You already have the topic text in hand, and want to help TWiki avoid
+        having to read it again,
+      1 You are providing a =$meta= parameter.
    * =$topic=    - Topic name, required, e.g. ='PrivateStuff'=
    * =$web=      - Web name, required, e.g. ='Sandbox'=
+   * =$meta=     - Meta-data object, as returned by =readTopic=. Optional.
+     If =undef=, but =$text= is defined, then access controls will be parsed
+     from =$text=. If defined, then metadata embedded in =$text= will be
+     ignored. This parameter is always ignored if =$text= is undefined.
+     Settings in =$meta= override =Set= settings in $text.
 A perl true result indicates that access is permitted.
 
+*Note* the wierd parameter order is due to compatibility constraints with
+earlier TWiki releases.
+
+*Tip* if you want, you can use this method to check your own access control types. For example, if you:
+   * Set ALLOWTOPICSPIN = IncyWincy
+in =ThatWeb.ThisTopic=, then a call to =checkAccessPermissions('SPIN', 'IncyWincy', undef, 'ThisTopic', 'ThatWeb', undef)= will return =true=.
+
 *Since:* TWiki::Plugins::VERSION 1.000 (27 Feb 2001)
 
 =cut
 
 sub checkAccessPermission {
-    my( $type, $user, $text, $topic, $web ) = @_;
+    my( $type, $user, $text, $topic, $web, $meta ) = @_;
     return 1 unless ( $user );
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
     $text = undef unless $text;
     $user = $TWiki::Plugins::SESSION->{users}->findUser( $user );
-    return $TWiki::Plugins::SESSION->{security}->checkAccessPermission
-      ( $type, $user, $text, undef, $topic, $web );
+    return $TWiki::Plugins::SESSION->{security}->checkAccessPermission(
+        $type, $user, $text, $meta, $topic, $web );
 }
 
 =pod
@@ -663,6 +701,7 @@ sub checkAccessPermission {
 =pod
 
 ---+++ getListOfWebs( $filter ) -> @webs
+
    * =$filter= - spec of web types to recover
 Gets a list of webs, filtered according to the spec in the $filter,
 which may include one of:
@@ -819,21 +858,24 @@ sub topicExists {
 
 =pod
 
----+++ checkTopicEditLock( $web, $topic ) -> ( $oopsUrl, $loginName, $unlockTime )
+---+++ checkTopicEditLock( $web, $topic, $script ) -> ( $oopsUrl, $loginName, $unlockTime )
+
 Check if a lease has been taken by some other user.
    * =$web= Web name, e.g. ="Main"=, or empty
    * =$topic= Topic name, e.g. ="MyTopic"=, or ="Main.MyTopic"=
 Return: =( $oopsUrl, $loginName, $unlockTime )= - The =$oopsUrl= for calling redirectCgiQuery(), user's =$loginName=, and estimated =$unlockTime= in minutes, or ( '', '', 0 ) if no lease exists.
+   * =$script= The script to invoke when continuing with the edit
 
 *Since:* TWiki::Plugins::VERSION 1.010 (31 Dec 2002)
 
 =cut
 
 sub checkTopicEditLock {
-    my( $web, $topic ) = @_;
+    my( $web, $topic, $script ) = @_;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
 
     ( $web, $topic ) = normalizeWebTopicName( $web, $topic );
+    $script ||= 'edit';
 
     my $lease = $TWiki::Plugins::SESSION->{store}->getLease( $web, $topic );
     if( $lease ) {
@@ -850,10 +892,11 @@ sub checkTopicEditLock {
                                                   $TWiki::Plugins::SESSION->{i18n}
                                                  );
             return( $session->getOopsUrl( 'leaseconflict',
-                                          def => 'active',
+                                          def => 'lease_active',
+					  keep => 1,   # Need to keep parameters across redirect
                                           web => $web,
                                           topic => $topic,
-                                          params => [ $wn, $past, $future ] ),
+                                          params => [ $wn, $past, $future, $script ] ),
                                           $who, $remain / 60 );
         }
     }
@@ -863,6 +906,7 @@ sub checkTopicEditLock {
 =pod
 
 ---+++ setTopicEditLock( $web, $topic, $lock )
+
    * =$web= Web name, e.g. ="Main"=, or empty
    * =$topic= Topic name, e.g. ="MyTopic"=, or ="Main.MyTopic"=
    * =$lock= 1 to lease the topic, 0 to clear the lease=
@@ -896,6 +940,7 @@ sub setTopicEditLock {
 =pod
 
 ---+++ saveTopic( $web, $topic, $meta, $text, $options ) -> $error
+
    * =$web= - web for the topic
    * =$topic= - topic name
    * =$meta= - reference to TWiki::Meta object
@@ -1017,6 +1062,7 @@ sub saveTopicText {
 =pod
 
 ---+++ moveTopic( $web, $topic, $newWeb, $newTopic )
+
    * =$web= source web - required
    * =$topic= source topic - required
    * =$newWeb= dest web
@@ -1065,6 +1111,7 @@ sub moveTopic {
 =pod
 
 ---+++ getRevisionInfo($web, $topic, $rev, $attachment ) -> ( $date, $user, $rev, $comment ) 
+
 Get revision info of a topic or attachment
    * =$web= - Web name, optional, e.g. ='Main'=
    * =$topic=   - Topic name, required, e.g. ='TokyoOffice'=
@@ -1212,6 +1259,7 @@ sub attachmentExists {
 =pod
 
 ---+++ readAttachment( $web, $topic, $name, $rev ) -> $data
+
    * =$web= - web for topic
    * =$topic= - topic
    * =$name= - attachment name
@@ -1258,6 +1306,7 @@ sub readAttachment {
 =pod
 
 ---+++ saveAttachment( $web, $topic, $attachment, $opts )
+
    * =$web= - web for topic
    * =$topic= - topic to atach to
    * =$attachment= - name of the attachment
@@ -1311,6 +1360,7 @@ sub saveAttachment {
 =pod
 
 ---+++ moveAttachment( $web, $topic, $attachment, $newWeb, $newTopic, $newAttachment )
+
    * =$web= source web - required
    * =$topic= source topic - required
    * =$attachment= source attachment - required
@@ -1417,6 +1467,7 @@ sub loadTemplate {
 =pod
 
 ---+++ expandTemplate( $def  ) -> $string
+
 Do a %TMPL:P{$def}%, only expanding the template (not expanding any variables other than %TMPL)
    * =$def= - template name
 Return: the text of the expanded template
@@ -1454,26 +1505,52 @@ sub writeHeader {
 
 =pod
 
----+++ redirectCgiQuery( $query, $url )
+---+++ redirectCgiQuery( $query, $url, $passthru )
 
 Redirect to URL
    * =$query= - CGI query object. Ignored, only there for compatibility. The session CGI query object is used instead.
    * =$url=   - URL to redirect to
-Return:             none, never returns
+   * =$passthru= - enable passthrough.
+
+Return:             none
+
+Print output to STDOUT that will cause a 302 redirect to a new URL.
+Nothing more should be printed to STDOUT after this method has been called.
+
+The =$passthru= parameter allows you to pass the parameters that were passed
+to the current query on to the target URL, as long as it is another URL on the
+same TWiki installation. If =$passthru= is set to a true value, then TWiki
+will save the current URL parameters, and then try to restore them on the
+other side of the redirect. Parameters are stored on the server in a cache
+file (see ={PassthroughDir} in =configure=).
+
+Note that if =$passthru= is set, then any parameters in =$url= will be lost
+when the old parameters are restored. if you want to change any parameter
+values, you will need to do that in the current CGI query before redirecting
+e.g.
+<verbatim>
+my $query = TWiki::Func::getCgiQuery();
+$query->param(-name => 'text', -value => 'Different text');
+TWiki::Func::redirectCgiQuery(
+  undef, TWiki::Func::getScriptUrl($web, $topic, 'edit'), 1);
+</verbatim>
+=$passthru= does nothing if =$url= does not point to a script in the current
+TWiki installation.
 
 *Since:* TWiki::Plugins::VERSION 1.000 (7 Dec 2002)
 
 =cut
 
 sub redirectCgiQuery {
-    my( $query, $url ) = @_;
+    my( $query, $url, $passthru ) = @_;
     ASSERT($TWiki::Plugins::SESSION) if DEBUG;
-    return $TWiki::Plugins::SESSION->redirect( $url );
+    return $TWiki::Plugins::SESSION->redirect( $url, $passthru );
 }
 
 =pod
 
 ---+++ addToHEAD( $id, $header )
+
 Adds =$header= to the HTML header (the <head> tag).
 This is useful for Plugins that want to include some javascript custom css.
    * =$id= - Unique ID to prevent the same HTML from being duplicated. Plugins should use a prefix to prevent name clashes (e.g EDITTABLEPLUGIN_JSCALENDAR)
@@ -1570,6 +1647,7 @@ sub internalLink {
 ---++ E-mail
 
 ---+++ sendEmail ( $text, $retries ) -> $error
+
    * =$text= - text of the mail, including MIME headers
    * =$retries= - number of times to retry the send (default 1)
 Send an e-mail specified as MIME format content. To specify MIME
@@ -1604,6 +1682,7 @@ sub sendEmail {
 =pod
 
 ---+++ wikiToEmail( $wikiName ) -> $email
+
    * =$wikiName= - wiki name of the user
 Get the e-mail address(es) of the named user. If the user has multiple
 e-mail addresses (for example, the user is a group), then the list will
@@ -1631,6 +1710,7 @@ sub wikiToEmail {
 =pod
 
 ---+++ expandVariablesOnTopicCreation ( $text ) -> $text
+
 Expand the limited set of variables that are always expanded during topic creation
    * =$text= - the text to process
 Return: text with variables expanded
@@ -1662,6 +1742,7 @@ sub expandVariablesOnTopicCreation {
 =pod
 
 ---++ Special handlers
+
 Special handlers can be defined to make functions in plugins behave as if they were built-in to TWiki.
 
 =cut
@@ -1669,6 +1750,7 @@ Special handlers can be defined to make functions in plugins behave as if they w
 =pod=
 
 ---+++ registerTagHandler( $var, \&fn, $syntax )
+
 Should only be called from initPlugin.
 
 Register a function to handle a simple variable. Handles both %<nop>VAR% and %<nop>VAR{...}%. Registered variables are treated the same as TWiki internal variables, and are expanded at the same time. This is a _lot_ more efficient than using the =commonTagsHandler=.
@@ -1707,6 +1789,12 @@ sub boo {
 would let you do this:
 =%<nop>EXEC{"ps -Af" silent="on"}%=
 
+Registered tags differ from tags implemented using the old TWiki approach (text substitution in =commonTagsHandler=) in the following ways:
+   * registered tags are evaluated at the same time as system tags, such as %SERVERTIME. =commonTagsHandler= is only called later, when all system tags have already been expanded (though they are expanded _again_ after =commonTagsHandler= returns).
+   * registered tag names can only contain alphanumerics and _ (underscore)
+   * registering a tag =FRED= defines both =%<nop>FRED{...}%= *and also* =%FRED%=.
+   * registered tag handlers *cannot* return another tag as their only result (e.g. =return '%<nop>SERVERTIME%';=). It won't work.
+
 =cut
 
 sub registerTagHandler {
@@ -1729,6 +1817,7 @@ sub registerTagHandler {
 =pod=
 
 ---+++ registerRESTHandler( $alias, \&fn, )
+
 Should only be called from initPlugin.
 
 Adds a function to the dispatch table of the REST interface 
@@ -1798,6 +1887,7 @@ sub registerRESTHandler {
 =pod
 
 ---+++ searchInWebContent($searchString, $web, \@topics, \%options ) -> \%map
+
 Search for a string in the content of a web. The search is over all content, including meta-data. Meta-data matches will be returned as formatted lines within the topic content (meta-data matches are returned as lines of the format %META:\w+{.*}%)
    * =$searchString= - the search string, in egrep format
    * =$web= - The web to search in
@@ -1927,7 +2017,7 @@ Return: String or precompiled regular expression matching as described below.
 *Since:* TWiki::Plugins::VERSION 1.020 (9 Feb 2004)
 
 __Note:__ TWiki internally precompiles several regular expressions to
-represent various string entities in an I18N-compatible manner. Plugins
+represent various string entities in an <nop>I18N-compatible manner. Plugins
 authors are encouraged to use these in matching where appropriate. The
 following are guaranteed to be present. Others may exist, but their use
 is unsupported and they may be removed in future TWiki versions.
@@ -1975,20 +2065,27 @@ sub getRegularExpression {
 Parse a web and topic name, supplying defaults as appropriate.
    * =$web= - Web name, identifying variable, or empty string
    * =$topic= - Topic name, may be a web.topic string, required.
-Return: the parsed Web/Topic pai
+Return: the parsed Web/Topic pair
 
 *Since:* TWiki::Plugins::VERSION 1.1
 
-| *Input* | *Return* |
-| <tt>( 'Web',  'Topic' )     </tt> | <tt>( 'Web',  'Topic' ) </tt> |
-| <tt>( '',     'Topic' )     </tt> | <tt>( 'Main', 'Topic' ) </tt> |
-| <tt>( '',     '' )          </tt> | <tt>( 'Main', 'WebHome' ) </tt> |
-| <tt>( '',     'Web/Topic' ) </tt> | <tt>( 'Web',  'Topic' ) </tt> |
-| <tt>( '',     'Web.Topic' ) </tt> | <tt>( 'Web',  'Topic' ) </tt> |
-| <tt>( 'Web1', 'Web2.Topic' )</tt> | <tt>( 'Web2', 'Topic' ) </tt> |
-| <tt>( '%MAINWEB%', 'Topic' )</tt> | <tt>( 'Main', 'Topic' ) </tt> |
-| <tt>( '%TWIKIWEB%', 'Topic' )</tt> | <tt>( 'TWiki', 'Topic' ) </tt> |
-where =Main= and =TWiki= are the web names set in $cfg{UsersWebName} and $cfg{SystemWebName} respectively.
+| *Input*                               | *Return*  |
+| <tt>( 'Web', 'Topic' ) </tt>          | <tt>( 'Web', 'Topic' ) </tt>  |
+| <tt>( '', 'Topic' ) </tt>             | <tt>( 'Main', 'Topic' ) </tt>  |
+| <tt>( '', '' ) </tt>                  | <tt>( 'Main', 'WebHome' ) </tt>  |
+| <tt>( '', 'Web/Topic' ) </tt>         | <tt>( 'Web', 'Topic' ) </tt>  |
+| <tt>( '', 'Web/Subweb/Topic' ) </tt>  | <tt>( 'Web/Subweb', 'Topic' ) </tt>  |
+| <tt>( '', 'Web.Topic' ) </tt>         | <tt>( 'Web', 'Topic' ) </tt>  |
+| <tt>( '', 'Web.Subweb.Topic' ) </tt>  | <tt>( 'Web/Subweb', 'Topic' ) </tt>  |
+| <tt>( 'Web1', 'Web2.Topic' )</tt>     | <tt>( 'Web2', 'Topic' ) </tt>  |
+
+Note that hierarchical web names (Web.SubWeb) are only available if hierarchical webs are enabled in =configure=.
+
+The symbols %<nop>USERSWEB%, %<nop>SYSTEMWEB%, %<nop>DOCWEB%, %<nop>MAINWEB% and %<nop>TWIKIWEB% can be used in the input to represent the web names set in $cfg{UsersWebName} and $cfg{SystemWebName}. For example:
+| *Input*                               | *Return* |
+| <tt>( '%<nop>USERSWEB%', 'Topic' )</tt>     | <tt>( 'Main', 'Topic' ) </tt>  |
+| <tt>( '%<nop>SYSTEMWEB%', 'Topic' )</tt>    | <tt>( 'TWiki', 'Topic' ) </tt>  |
+| <tt>( '', '%<nop>DOCWEB%.Topic' )</tt>    | <tt>( 'TWiki', 'Topic' ) </tt>  |
 
 =cut
 
@@ -2214,7 +2311,7 @@ sub formatGmTime {
 
 ---+++ getDataDir( ) -> $dir
 
-*DEPRECATED* since 1.1 - use the [[#Functions_Content_Handling][content handling functions]] to manipulate topics instead
+*DEPRECATED* since 1.1 - use the "Webs, Topics and Attachments" functions to manipulate topics instead
 
 Get data directory (topic file root)
 
@@ -2234,7 +2331,7 @@ sub getDataDir {
 
 ---+++ getPubDir( ) -> $dir
 
-*DEPRECATED* since 1.1 - use the [[#Functions_Content_Handling][content handling functions]] to manipulateattachments instead
+*DEPRECATED* since 1.1 - use the "Webs, Topics and Attachments" functions to manipulateattachments instead
 
 Get pub directory (file attachment root). Attachments are in =$dir/Web/TopicName=
 
