@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 2001-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 2001-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -40,8 +40,7 @@ use TWiki::Attrs ();
 use TWiki::Time ();
 
 # Used to generate unique placeholders for when we lift blocks out of the
-# text during rendering. SMELL: a horrible hack to get around the horrible
-# way the rendering engine works.
+# text during rendering. 
 use vars qw( $placeholderMarker );
 $placeholderMarker = 0;
 
@@ -92,6 +91,11 @@ sub new {
     $this->{NEWLINKSYMBOL} =
       $session->{prefs}->getPreferencesValue('NEWTOPICLINKSYMBOL')
         || CGI::sup('?');
+    $this->{NEWLINKFORMAT} =
+      $session->{prefs}->getPreferencesValue('NEWLINKFORMAT')
+        || '<span class="twikiNewLink">$text<a href="%SCRIPTURLPATH{"edit"}%/$web/$topic?topicparent=%WEB%.%TOPIC%" '.
+	   'rel="nofollow" title="%MAKETEXT{"Create this topic"}%">'.
+           '$linksymbol</a></span>';
     # tooltip init
     $this->{LINKTOOLTIPINFO} =
       $session->{prefs}->getPreferencesValue('LINKTOOLTIPINFO')
@@ -105,6 +109,7 @@ sub new {
 =pod
 
 ---++ ObjectMethod renderParent($web, $topic, $meta, $params) -> $text
+
 Render parent meta-data
 
 =cut
@@ -118,6 +123,8 @@ sub renderParent {
     my $usesep =      $ah->{separator} || ' &gt; ';
     my $format =      $ah->{format} || '[[$web.$topic][$topic]]';
 
+    return '' unless $web && $topic;
+
     my %visited;
     $visited{$web.'.'.$topic} = 1;
 
@@ -160,6 +167,7 @@ sub renderParent {
 =pod
 
 ---++ ObjectMethod renderMoved($web, $topic, $meta, $params) -> $text
+
 Render moved meta-data
 
 =cut
@@ -210,6 +218,7 @@ sub renderMoved {
 =pod
 
 ---++ ObjectMethod renderFormField($web, $topic, $meta, $params) -> $text
+
 Render meta-data for a single formfield
 
 =cut
@@ -219,10 +228,17 @@ sub renderFormField {
     my $text = '';
     my $name = $attrs->{name};
     $text = renderFormFieldArg( $meta, $name ) if( $name );
+    my $newline = $attrs->{newline};
+    if ( defined $newline ) {
+      $newline =~ s/\$n/\n/gos;
+    } else {
+      $newline = "<br />";
+    }
+    my $bar = $attrs->{bar} || "&#124;";
     # change any new line character sequences to <br />
-    $text =~ s/\r?\n/ <br \/> /gos;
+    $text =~ s/\r?\n/$newline/gos;
     # escape "|" to HTML entity
-    $text =~ s/\|/\&\#124;/gos;
+    $text =~ s/\|/$bar/gos;
     return $text;
 }
 
@@ -368,7 +384,6 @@ sub makeAnchorName {
     }
 
     # strip out potential links so they don't get rendered.
-    # SMELL: Screws up header rendering.
     # remove double bracket link
     $anchorName =~ s/\s*\[\s*\[.*?\]\s*\[(.*?)\]\s*\]/$1/go;
     $anchorName =~ s/\s*\[\s*\[\s*(.*?)\s*\]\s*\]/$1/go;
@@ -446,8 +461,8 @@ sub _linkToolTipInfo {
 
 Generate a link. 
 
-SMELL: why can topic be spaced out? is this to support auto squishing of [[Spaced Topic Naming]]?
-and [[lowercase Spaced Topic Naming]]
+Note: Topic names may be spaced out. Spaced out names are converted to <nop>WikWords,
+for example, "spaced topic name" points to "SpacedTopicName".
    * =$theWeb= - the web containing the topic
    * =$theTopic= - the topic to be lunk
    * =$theLinkText= - text to use for the link
@@ -518,13 +533,12 @@ sub _renderWikiWord {
                                        $theTopic, $theLinkText, $theAnchor);
     }
     if( $doLinkToMissingPages ) {
-        my @topics = ( $theTopic );
         # CDot: disabled until SuggestSingularNotPlural is resolved
         # if ($singular && $singular ne $theTopic) {
         #     #unshift( @topics, $singular);
         # }
-        return _renderNonExistingWikiWord($this, $theWeb, \@topics,
-                                          $theLinkText, $theAnchor);
+        return _renderNonExistingWikiWord($this, $theWeb, $theTopic,
+                                          $theLinkText);
     }
     if( $doKeepWeb ) {
         return $theWeb.'.'.$theLinkText;
@@ -536,13 +550,21 @@ sub _renderWikiWord {
 sub _renderExistingWikiWord {
     my ($this, $web, $topic, $text, $anchor) = @_;
 
+    my $currentWebHome = '';
+    $currentWebHome = 'twikiCurrentWebHomeLink ' if (($web eq $this->{session}->{webName}) &&
+                                      ($topic eq $TWiki::cfg{HomeTopicName} ));
+
+    my $currentTopic = '';
+    $currentTopic = 'twikiCurrentTopicLink ' if (($web eq $this->{session}->{webName}) &&
+                                       ($topic eq $this->{session}->{topicName}));
+
     my @attrs;
     my $href = $this->{session}->getScriptUrl( 0, 'view', $web, $topic );
     if( $anchor ) {
         $anchor = $this->makeAnchorName( $anchor );
-        push( @attrs, class => 'twikiAnchorLink', href => $href.'#'.$anchor );
+        push( @attrs, class => $currentTopic.$currentWebHome.'twikiAnchorLink', href => $href.'#'.$anchor );
     } else {
-        push( @attrs, class => 'twikiLink', href => $href );
+        push( @attrs, class => $currentTopic.$currentWebHome.'twikiLink', href => $href );
     }
     my $tooltip = $this->_linkToolTipInfo( $web, $topic );
     push( @attrs, title => $tooltip ) if( $tooltip );
@@ -556,41 +578,16 @@ sub _renderExistingWikiWord {
 }
 
 sub _renderNonExistingWikiWord {
-    my ($this, $theWeb, $theTopic, $theLinkText, $theAnchor) = @_;
-    my $ans;
+    my ($this, $theWeb, $theTopic, $theLinkText) = @_;
 
-    $ans = $theLinkText;
-
-    if (ref $theTopic && ref $theTopic eq 'ARRAY') {
-        my $num = 1;
-        my @posse = ();
-        foreach my $t(@{ $theTopic }) {
-            next if ! $t;
-            push( @posse,
-                  CGI::a(
-                      { href => $this->{session}->getScriptUrl(
-                          0, 'edit', $theWeb, $t,
-                          topicparent => $this->{session}->{webName}.'.'.
-                            $this->{session}->{topicName} ),
-                        rel => 'nofollow',
-                        title => ($this->{session}->{i18n}->maketext('Create this topic'))
-                    },
-                    $this->{NEWLINKSYMBOL} x $num ));
-            $num++;
-        }
-        $ans .= join( ' ', @posse );
-    } else {
-        $ans .= CGI::a( { href=>$this->{session}->getScriptUrl(
-            0, 'edit', $theWeb, $theTopic,
-            topicparent => $this->{session}->{webName}.'.'.
-              $this->{session}->{topicName} ),
-                          rel=>'nofollow',
-                          title=>($this->{session}->{i18n}->maketext('Create this topic'))
-                    },
-                    $this->{NEWLINKSYMBOL} );
-    }
-    return CGI::span( { class=>'twikiNewLink' },
-                      $ans );
+    my $ans = $this->{NEWLINKFORMAT};
+    $ans =~ s/\$web/$theWeb/g;
+    $ans =~ s/\$topic/$theTopic/g;
+    $ans =~ s/\$text/$theLinkText/g;
+    $ans =~ s/\$linksymbol/$this->{NEWLINKSYMBOL}/g;
+    $ans = $this->{session}->handleCommonTags($ans, 
+	$this->{session}{webName}, $this->{session}{topicName});
+    return $ans;
 }
 
 # _handleWikiWord is called by the TWiki Render routine when it sees a 
@@ -656,36 +653,26 @@ sub _handleSquareBracketedLink {
     my( $this, $web, $topic, $link, $text ) = @_;
 
     # Strip leading/trailing spaces
-    $link =~ s/^\s*//;
-    $link =~ s/\s*$//;
+    $link =~ s/^\s+//;
+    $link =~ s/\s+$//;
 
-    # Be friendly to file:-links, never alter them no matter what
-    if( $link =~ /^file\:/ ) {
-          # Prevent automatic WikiWord or CAPWORD linking in explicit links
-          $link =~ s/(?<=[\s\(])($TWiki::regex{wikiWordRegex}|[$TWiki::regex{upperAlpha}])/<nop>$1/go;
-          return $this->_externalLink( $link, $text );
-    }
-
-    # Spot other full explicit URLs
-    # (explicit external [[$link][$text]]-style, that can be handled directly)
-    if( $link =~ /^$TWiki::regex{linkProtocolPattern}\:/ ) {
-        if (defined $text && !($link eq $text)) {
+    # Explicit external [[$link][$text]]-style can be handled directly
+    if( $link =~ m!^($TWiki::regex{linkProtocolPattern}\:|/)! ) {
+        if (defined $text) {
+            # [[][]] style - protect text:
             # Prevent automatic WikiWord or CAPWORD linking in explicit links
-            $link =~ s/(?<=[\s\(])($TWiki::regex{wikiWordRegex}|[$TWiki::regex{upperAlpha}])/<nop>$1/go;
-            return $this->_externalLink( $link, $text );
-        }
-    }
-
-    # Spot URLs
-    if( $link =~ /^$TWiki::regex{linkProtocolPattern}\:/ ||
-          $link =~ /^\// ) {
-        # URL, absolute or relative
-        if ( $link =~ /^(\S+)\s+(.*)$/ ) {
-            # '[[URL#anchor display text]]' link:
-            $link = $1;
-            $text = $2;
             $text =~ s/(?<=[\s\(])($TWiki::regex{wikiWordRegex}|[$TWiki::regex{upperAlpha}])/<nop>$1/go;
         }
+        else {
+            # [[]] style - take care for legacy:
+            # Prepare special case of '[[URL#anchor display text]]' link
+            if ( $link =~ /^(\S+)\s+(.*)$/ ) {
+                # '[[URL#anchor display text]]' link:
+                $link = $1;
+                $text = $2;
+                $text =~ s/(?<=[\s\(])($TWiki::regex{wikiWordRegex}|[$TWiki::regex{upperAlpha}])/<nop>$1/go;
+            }
+        }
         return $this->_externalLink( $link, $text );
     }
 
@@ -713,24 +700,6 @@ sub _handleSquareBracketedLink {
 
     $topic = $link if( $link );
 
-    if( $TWiki::cfg{EnableHierarchicalWebs} ) {
-        # look up the leading path components to see
-        # if they form a valid web path.
-        my @topica = split( /\./, $topic );
-        my @weba;
-        while( @topica && $this->{session}->{store}->webExists(
-            join('.', @weba, $topica[0]))) {
-            push(@weba, shift(@topica));
-        }
-
-        $web = join('.', @weba) if scalar(@weba);
-        $topic = join('', @topica);
-    } else {
-        if( $topic =~ s/^($TWiki::regex{webNameBaseRegex}|$TWiki::regex{defaultWebNameRegex})\.//) {
-            $web = $1;
-        }
-        $topic =~ s/\.//g;
-    }
     # Topic defaults to the current topic
     ($web, $topic) = $this->{session}->normalizeWebTopicName( $web, $topic );
 
@@ -751,12 +720,18 @@ sub _externalLink {
     my $opt = '';
     if( $url =~ /^mailto:/i ) {
         if( $TWiki::cfg{AntiSpam}{EmailPadding} ) {
-            $url =~ s/(@\w*)/$1$TWiki::cfg{AntiSpam}{EmailPadding}/;
-
+            $url =~  s/(\@[\w\_\-\+]+)(\.)/$1$TWiki::cfg{AntiSpam}{EmailPadding}$2/;
+            if ($text) {
+                $text =~ s/(\@[\w\_\-\+]+)(\.)/$1$TWiki::cfg{AntiSpam}{EmailPadding}$2/;
+            }
         }
         if( $TWiki::cfg{AntiSpam}{HideUserDetails} ) {
-            # Much harder obfuscation scheme
+            # Much harder obfuscation scheme. For link text we only encode '@'
+            # See also Item2928 and Item3430 before touching this
             $url =~ s/(\W)/'&#'.ord($1).';'/ge;
+            if ($text) {
+                $text =~ s/\@/'&#'.ord('@').';'/ge;
+            }
         }
     } else {
         $opt = ' target="_top"';
@@ -900,8 +875,9 @@ sub getRenderedVersion {
     my $removedVerbatim = {};
     my $removedLiterals = {};
 
-    $text = $this->takeOutBlocks( $text, 'literal', $removedLiterals );
+    # verbatim before literal - see Item3431
     $text = $this->takeOutBlocks( $text, 'verbatim', $removedVerbatim );
+    $text = $this->takeOutBlocks( $text, 'literal', $removedLiterals );
 
     $text = $this->takeOutProtected( $text, qr/<\?([^?]*)\?>/s,
                                      $removedComments );
@@ -1120,8 +1096,8 @@ sub getRenderedVersion {
     $text =~ s/(^|\s)\!\[\[/$1\[<nop>\[/gm;
     # Spaced-out Wiki words with alternative link text
     # i.e. [[$1][$3]]
-    $text =~ s/\[\[([^\]\n]+)\](\[([^\]\n]+)\])?\]/$this->_handleSquareBracketedLink($theWeb,$theTopic,$1,$3)/ge;
-
+    $text =~ s/\[\[([^\]\[\n]+)\](\[([^\]\n]+)\])?\]/$this->_handleSquareBracketedLink($theWeb,$theTopic,$1,$3)/ge;
+    
     unless( TWiki::isTrue( $prefs->getPreferencesValue('NOAUTOLINK')) ) {
         # Handle WikiWords
         $text = $this->takeOutBlocks( $text, 'noautolink', $removed );
@@ -1135,18 +1111,18 @@ sub getRenderedVersion {
     $plugins->endRenderingHandler( $text );
 
     # replace verbatim with pre in the final output
-    $this->putBackBlocks( \$text, $removedVerbatim, 'verbatim', 'pre', \&verbatimCallBack );
-    $this->putBackBlocks( \$text, $removedLiterals, 'literal', '');
-
+    $this->putBackBlocks( \$text, $removedVerbatim,
+                          'verbatim', 'pre', \&verbatimCallBack );
     $text =~ s|\n?<nop>\n$||o; # clean up clutch
 
-    # Only put script sections back if they are allowed by options
-    $this->putBackProtected( \$text, $removedScript )
-      if $TWiki::cfg{AllowInlineScript};
+    $this->putBackProtected( \$text, $removedScript, \&_filterScript );
+    $this->putBackBlocks( \$text, $removedLiterals,
+                          'literal', '', \&_filterLiteral );
 
     $this->putBackProtected( \$text, $removedHead );
     $this->putBackProtected( \$text, $removedComments );
-    $this->{session}->{client}->endRenderingHandler( $text );
+
+    $this->{session}->{loginManager}->endRenderingHandler( $text );
 
     $plugins->postRenderingHandler( $text );
     return $text;
@@ -1171,6 +1147,20 @@ sub verbatimCallBack {
     return TWiki::entityEncode( $val );
 }
 
+# Only put script and literal sections back if they are allowed by options
+sub _filterLiteral {
+    my $val = shift;
+    return $val if( $TWiki::cfg{AllowInlineScript} );
+    return CGI::span({class => 'twikiAlert'},
+                     '&lt;literal&gt; is not allowed on this site');
+}
+
+sub _filterScript {
+    my $val = shift;
+    return $val if( $TWiki::cfg{AllowInlineScript} );
+    return CGI::span({class => 'twikiAlert'},
+                     '&lt;script&gt; is not allowed on this site');
+}
 
 =pod
 
@@ -1222,17 +1212,19 @@ sub TML2PlainText {
 
     # Format e-mail to add spam padding (HTML tags removed later)
     $text =~ s/$STARTWW((mailto\:)?[a-zA-Z0-9-_.+]+@[a-zA-Z0-9-_.]+\.[a-zA-Z0-9-_]+)$ENDWW/$this->_mailLink( $1 )/gem;
-    $text =~ s/<!--.*?-->//gs;     # remove all HTML comments
-    $text =~ s/<[^>]*>//g;              # remove all HTML tags
+    $text =~ s/<!--.*?-->//gs;          # remove all HTML comments
+    $text =~ s/<(?!nop)[^>]*>//g;       # remove all HTML tags except <nop>
     $text =~ s/\&[a-z]+;/ /g;           # remove entities
     if( $opts =~ /nohead/ ) {
         # skip headings on top
         while( $text =~ s/^\s*\-\-\-+\+[^\n\r]*// ) {}; # remove heading
     }
-    # keep only link text of [[][]]
-    $text =~ s/\[\[([^\]]*\]\[|[^\s]*\s)(.*?)\]\]/$2/g;
+    # keep only link text of [[prot://uri.tld/ link text]] or [[][]]
+    $text =~ s/\[\[$TWiki::regex{linkProtocolPattern}\:([^\s<>"]+[^\s*.,!?;:)<|])\s+(.*?)\]\]/$3/g;
+    $text =~ s/\[\[([^\]]*\]\[)(.*?)\]\]/$2/g;
     # remove "Web." prefix from "Web.TopicName" link
     $text =~ s/$STARTWW(($TWiki::regex{webNameRegex})\.($TWiki::regex{wikiWordRegex}|$TWiki::regex{abbrevRegex}))/$3/g;
+    $text =~ s/<[^>]*>//g;              # remove all HTML tags
     $text =~ s/[\[\]\*\|=_\&\<\>]/ /g;  # remove Wiki formatting chars
     $text =~ s/^\-\-\-+\+*\s*\!*/ /gm;  # remove heading formatting and hbar
     $text =~ s/[\+\-]+/ /g;             # remove special chars
@@ -1317,7 +1309,11 @@ sub makeTopicSummary {
     $nchar = $MINTRUNC if( $nchar < $MINTRUNC );
     $htext =~ s/^(.{$nchar}.*?)($TWiki::regex{mixedAlphaNumRegex}).*$/$1$2 \.\.\./s;
 
-    # newline conversion to permit embedding in TWiki tables
+    # We do not want the summary to contain any $variable that formatted
+    # searches can interpret to anything (Item3489).
+    # Especially new lines (Item2496)
+    # To not waste performance we simply replace $ by $<nop>
+    $htext =~ s/\$/\$<nop>/g;
     $htext =~ s/\s+/ /g;
 
     return $this->protectPlainText( $htext );
@@ -1361,22 +1357,24 @@ sub _replaceBlock {
 
 =pod
 
----++ ObjectMethod putBackProtected( \$text, \%map, $tag, $newtag, $callBack ) -> $text
+---++ ObjectMethod putBackProtected( \$text, \%map, $callback ) -> $text
 
 Return value: $text with blocks added back
    * =\$text= - reference to text to process
    * =\%map= - map placeholders to blocks removed by takeOutBlocks
+   * =$callback= - Reference to function to call on each block being inserted (optional)
 
 Reverses the actions of takeOutProtected.
 
 =cut
 
 sub putBackProtected {
-    my( $this, $text, $map ) = @_;
+    my( $this, $text, $map, $callback ) = @_;
     ASSERT($this->isa( 'TWiki::Render')) if DEBUG;
 
     foreach my $placeholder ( keys %$map ) {
         my $val = $map->{$placeholder}{text};
+        $val = &$callback( $val ) if( defined( $callback ));
         $$text =~ s/<!--$TWiki::TranslationToken$placeholder$TWiki::TranslationToken-->/$val/;
         delete( $map->{$placeholder} );
     }
@@ -1509,10 +1507,12 @@ sub putBackBlocks {
             delete( $map->{$placeholder} );
         }
     }
-    
-	if ($newtag eq '') {
-		$$text =~ s/&#60;\/?$tag&#62;//ge;
-	}
+    # This was added by Sven in 7219, but without any clear explanation
+    # or tests. It removes perfectly valid content from inside verbatim
+    # tags, and causes the unit tests for <literal> to fail.
+    #	if ($newtag eq '') {
+    #		$$text =~ s/&#60;\/?$tag&#62;//g;
+    #	}
 }
 
 =pod
@@ -1529,12 +1529,13 @@ Obtain and render revision info for a topic.
    | =$web= | the web name |
    | =$topic= | the topic name |
    | =$rev= | the rev number |
-   | =$date= | the date of the rev (no time) |
-   | =$time= | the full date and time of the rev |
    | =$comment= | the comment |
    | =$username= | the login of the saving user |
    | =$wikiname= | the wikiname of the saving user |
    | =$wikiusername= | the web.wikiname of the saving user |
+   | =$date= | the date of the rev (no time) |
+   | =$time= | the time of the rev |
+   | =$min=, =$sec=, etc. | Same date format qualifiers as GMTIME |
 
 =cut
 
@@ -1568,6 +1569,10 @@ sub renderRevisionInfo {
     $value =~ s/\$rev/$rev/gi;
     $value =~ s/\$time/TWiki::Time::formatTime($date, '$hour:$min:$sec')/gei;
     $value =~ s/\$date/TWiki::Time::formatTime($date, '$day $mon $year')/gei;
+    $value =~ s/(\$(rcs|http|email|iso))/TWiki::Time::formatTime($date, $1 )/gei;
+    if( $value =~ /\$(sec|min|hou|day|wday|dow|week|mo|ye|epoch|tz)/ ) {
+        $value = TWiki::Time::formatTime( $date, $value );
+    }
     $value =~ s/\$comment/$comment/gi;
     $value =~ s/\$username/$un/gi;
     $value =~ s/\$wikiname/$wn/gi;
@@ -1579,6 +1584,7 @@ sub renderRevisionInfo {
 =pod
 
 ---++ ObjectMethod summariseChanges($user, $web, $topic, $orev, $nrev, $tml) -> $text
+
    * =$user= - user (null to ignore permissions)
    * =$web= - web
    * =$topic= - topic
@@ -1598,7 +1604,7 @@ In non-tml, lines are truncated to 70 characters. Differences are shown using +
 sub summariseChanges {
     my( $this, $user, $web, $topic, $orev, $nrev, $tml ) = @_;
     ASSERT($this->isa( 'TWiki::Render')) if DEBUG;
-    ASSERT($user->isa( 'TWiki::User')) if DEBUG;
+    #ASSERT($user->isa( 'TWiki::User')) if DEBUG;
     my $summary = '';
     my $store = $this->{session}->{store};
 
@@ -1726,6 +1732,7 @@ sub forEachLine {
 =pod
 
 ---++ StaticMethod replaceTopicReferences( $text, \%options ) -> $text
+
 Callback designed for use with forEachLine, to replace topic references.
 \%options contains:
    * =oldWeb= => Web of reference to replace
@@ -1776,6 +1783,7 @@ sub replaceTopicReferences {
 =pod
 
 ---++ StaticMethod replaceWebReferences( $text, \%options ) -> $text
+
 Callback designed for use with forEachLine, to replace web references.
 \%options contains:
    * =oldWeb= => Web of reference to replace
@@ -1883,6 +1891,13 @@ sub _replaceInternalRefs {
 Parse the arguments to a $formfield specification and extract
 the relevant formfield from the given meta data.
 
+   * =args= string containing name of form field
+   
+In addition to the name of a field =args= can be appended with a commas
+followed by a string format (\d+)([,\s*]\.\.\.)?). This supports the formatted
+search function $formfield and is used to shorten the returned string or a 
+hyphenated string.        
+
 =cut
 
 sub renderFormFieldArg {
@@ -1902,9 +1917,13 @@ sub renderFormFieldArg {
         if( $name =~ /^($field->{name}|$title)$/ ) {
             $value = $field->{value};
             $value = '' unless defined( $value );
-            $value =~ s/^\s*(.*?)\s*$/$1/go;
-            $value = breakName( $value, $breakArgs );
-
+            if ( $breakArgs ) {
+                $value =~ s/^\s*(.*?)\s*$/$1/go;
+                $value = breakName( $value, $breakArgs );
+            }
+            # Item3489, Item2837. Prevent $vars in formfields from
+            # being expanded in formatted searches.
+            $value =~ s/\$(n|nop|quot|percnt|dollar)/\$<nop>$1/go;
             return $value;
         }
     }
@@ -1914,6 +1933,7 @@ sub renderFormFieldArg {
 =pod
 
 ---++ StaticMethod breakName( $text, $args) -> $text
+
    * =$text= - text to "break"
    * =$args= - string of format (\d+)([,\s*]\.\.\.)?)
 Hyphenates $text every $1 characters, or if $2 is "..." then shortens to
