@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 2002-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 2002-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -88,6 +88,7 @@ sub new {
 =pod
 
 ---++ ObjectMethod finish
+
 Complete processing after the client's HTTP request has been responded
 to.
    1 breaking circular references to allow garbage collection in persistent
@@ -97,7 +98,6 @@ to.
 
 sub finish {
     my $this = shift;
-    
     $this->{IMPL}->finish();
 }
 
@@ -144,12 +144,83 @@ sub readTopic {
     my $meta = new TWiki::Meta( $this->{session}, $web, $topic);
     $this->extractMetaData( $meta, \$text );
 
-	# Override meta with that blended from pub.
-	$meta = $this->_extractMetaDataAutoAttachments($user, $web, $topic, $version, $meta );
+    # Override meta with that blended from pub.
+    if ($TWiki::cfg{AutoAttachPubFiles} &&
+        $web eq $this->{session}{webName} && # only check the currently requested topic
+        $topic eq $this->{session}{topicName}) {
+
+      my @knownAttachments = $meta->find('FILEATTACHMENT');
+      my @attachmentsFoundInPub = _findAttachments($this, $web, $topic, \@knownAttachments);
+      $meta->putAll('FILEATTACHMENT', @attachmentsFoundInPub) if @attachmentsFoundInPub;
+    }
 
     return( $meta, $text );
 }
 
+=pod 
+
+---++ ObjectMethod _findAttachments($session, $web, $topic, $knownAttachments) -> @attachmentsFoundInPub
+
+Synchronise the attachment list with what's actually on disk Returns an ARRAY
+of FILEATTACHMENTs. These can be put in the new meta using
+meta->put('FILEATTACHMENTS', $tree) 
+
+This function is only called when the AutoAttachPubFiles configuration option is set.
+
+IDEA On Windows machines where the underlying filesystem can store arbitary
+meta data against files, this might replace/fulfil the COMMENT purpose
+
+TODO consider logging when things are added to metadata
+
+=cut
+
+sub _findAttachments {
+    my ($this, $web, $topic, $attachmentsKnownInMeta) = @_;
+    my $session = $this->{session};
+    ASSERT($session->isa( 'TWiki' )) if DEBUG;
+
+    my $store = $this;
+
+    my %filesListedInPub = $store->getAttachmentList($web, $topic);
+    my %filesListedInMeta = ();
+
+    # You need the following lines if you want metadata to supplement
+    # the filesystem
+    if (defined $attachmentsKnownInMeta) {
+      %filesListedInMeta =
+        TWiki::Meta::indexByKey('name', @$attachmentsKnownInMeta);
+    }
+
+    foreach my $file (keys %filesListedInPub) {
+        if ($filesListedInMeta{$file}) {
+            # Bring forward any missing yet wanted attributes
+            #SMELL: this will over-write (empty) any meta data field not listed here :( 
+            foreach my $field qw(comment attr user version) {
+              if ($filesListedInMeta{$file}{$field}) {
+                $filesListedInPub{$file}{$field} =
+                $filesListedInMeta{$file}{$field};
+              }
+            }
+        # Develop:Bugs.Item452 - WHY IS USER STILL WRONG?
+      }
+    }
+
+    # A comparison of the keys of the $filesListedInMeta and %filesListedInPub 
+    # would show files that were in Meta but have disappeared from Pub.
+
+    # SMELL Meta really ought index its attachments in a hash by attachment
+    # name but this is not the case
+    #
+    # SMELL: Do not change this from array to hash, you would lose the
+    # proper attachment sequence 
+    #
+    my @deindexedBecauseMetaDoesnotIndexAttachments =
+      TWiki::Meta::deindexKeyed(%filesListedInPub);
+
+    return @deindexedBecauseMetaDoesnotIndexAttachments;
+}
+
+
 =pod
 
 ---++ ObjectMethod readTopicRaw( $user, $web, $topic, $version ) ->  $topicText
@@ -271,16 +342,21 @@ sub moveAttachment {
         $this->{session}->writeLog(
             'move',
             $fileAttachment->{movefrom}.' moved to '.
-              $fileAttachment->{movedto}, $user );
+              $fileAttachment->{movedto}, $user->webDotWikiName() );
     } finally {
         $this->unlockTopic( $user, $oldWeb, $oldTopic );
         $this->unlockTopic( $user, $newWeb, $newTopic );
     };
+
+    # alert plugins of attachment move
+    $this->{session}->{plugins}->afterRenameHandler( $oldWeb, $oldTopic, $oldAttachment,
+                                                     $newWeb, $newTopic, $newAttachment );
 }
 
 =pod
 
 ---++ ObjectMethod getAttachmentStream( $user, $web, $topic, $attName ) -> \*STREAM
+
    * =$user= - the user doing the reading, or undef if no access checks
    * =$web= - The web
    * =$topic= - The topic
@@ -343,72 +419,12 @@ sub attachmentExists {
     return $handler->storedDataExists();
 }
 
-# _findAttachments($session, $web, $topic, $knownAttachments)
-# Synchronise the attachment list with what's actually on disk
-# Returns an ARRAY of FILEATTACHMENTs- these can be put in the new meta using meta->put('FILEATTACHMENTS', $tree)
-#IDEA On Windows machines where the underlying filesystem can store
-#arbitary meta data against files, this might replace/fulfil the COMMENT
-# purpose
-#TODO consider logging when things are added to metadata
-
-
-sub _findAttachments {
-    my ($this, $web, $topic, $attachmentsKnownInMeta) = @_;
-    my $session = $this->{session};
-    ASSERT($session->isa( 'TWiki' )) if DEBUG;
-
-    my $store = $this;
-
-    my %filesListedInPub = $store->getAttachmentList($web, $topic);
-	my %filesListedInMeta = ();
-
-    # You need the following lines if you want metadata to supplement
-    # the filesystem	
-	if (defined $attachmentsKnownInMeta) {
-		%filesListedInMeta =
-          TWiki::Meta::indexByKey('name', @$attachmentsKnownInMeta);
-	}
-
-    # Please retain following print until this feature is out of beta
-#    	print "Topic: $web.$topic:\n";
-#    	use Data::Dumper;
-#    	$this->{session}->writeDebug("In Meta:".Dumper(\%filesListedInMeta). "\n\nIn Pub:\n".Dumper(\%filesListedInPub));
-
-    foreach my $file (keys %filesListedInPub) {
-        if ($filesListedInMeta{$file}) {
-            # Bring forward any missing yet wanted attributes
-            #SMELL: this will over-write (empty) any meta data field not listed here :( 
-            foreach my $field qw(comment attr user version) {
-              if ($filesListedInMeta{$file}{$field}) {
-#              		$this->{session}->writeDebug("$file: bringing forward field $field: was ".$filesListedInPub{$file}{$field});
-              		$filesListedInPub{$file}{$field} =
- 		            $filesListedInMeta{$file}{$field};
-# 		            $this->{session}->writeDebug(" now: ".$filesListedInPub{$file}{$field}."\n");
-              }
-            }
-         	# Develop:Bugs.Item452 - WHY IS USER STILL WRONG?
-	     }
-    }
-
-    # Please retain following print until this feature is out of beta
-    #	use Data::Dumper;
-# $this->{session}->writeDebug("Result:".Dumper(\%filesListedInPub)."\n");
-
-	# A comparison of the keys of the $filesListedInMeta and %filesListedInPub 
-	# would show files that were in Meta but have disappeared from Pub.
-		
-	# SMELL Meta really ought index its attachments in a hash by attachment
-    # name but this is not the case
-	# SMELL so fit the interface and return an ugly array instead
-	my @deindexedBecauseMetaDoesnotIndexAttachments =
-      TWiki::Meta::deindexKeyed(%filesListedInPub);
-
-	return @deindexedBecauseMetaDoesnotIndexAttachments;
-}
-
 =pod
+
 ---++ ObjectMethod _removeAutoAttachmentsFromMeta
+
 This is where we are going to remove from meta any entry that is marked as an automatic attachment.
+
 =cut
 
 sub _removeAutoAttachmentsFromMeta {
@@ -478,6 +494,10 @@ sub moveTopic {
         my $new = $newWeb.'.'.$newTopic;
         $this->{session}->writeLog( 'rename', $old, "moved to $new", $user );
     }
+
+    # alert plugins of topic move
+    $this->{session}->{plugins}->afterRenameHandler( $oldWeb, $oldTopic, '',
+                                                     $newWeb, $newTopic, '' );
 }
 
 # update .changes for statistics and mailer
@@ -546,6 +566,9 @@ sub moveWeb {
     if( $TWiki::cfg{Log}{rename} ) {
         $this->{session}->writeLog( 'renameweb', $oldWeb, 'moved to '.$newWeb, $user );
     }
+
+    # alert plugins of web move
+    $this->{session}->{plugins}->afterRenameHandler( $oldWeb, '', '', $newWeb, '', '' );
 }
 
 =pod
@@ -675,6 +698,7 @@ sub getRevisionDiff {
 =pod
 
 ---++ ObjectMethod getRevisionInfo($web, $topic, $rev, $attachment) -> ( $date, $user, $rev, $comment )
+
 Get revision info of a topic.
    * =$web= Web name, optional, e.g. ='Main'=
    * =$topic= Topic name, required, e.g. ='TokyoOffice'=
@@ -717,6 +741,7 @@ sub getRevisionInfo {
 =pod
 
 ---++ StaticMethod dataEncode( $uncoded ) -> $coded
+
 Encode meta-data fields, escaping out selected characters. The encoding
 is chosen to avoid problems with parsing the attribute values, while
 minimising the number of characters encoded so searches can still work
@@ -739,6 +764,7 @@ sub dataEncode {
 =pod
 
 ---++ StaticMethod dataDecode( $encoded ) -> $decoded
+
 Decode escapes in a string that was encoded using dataEncode
 
 The encoding has to be exported because TWiki (and plugins) use
@@ -792,6 +818,7 @@ sub _singleKey {
 =pod
 
 ---++ ObjectMethod saveTopic( $user, $web, $topic, $text, $meta, $options  )
+
    * =$user= - user doing the saving (object)
    * =$web= - web for topic
    * =$topic= - topic to atach to
@@ -817,8 +844,8 @@ sub saveTopic {
     ASSERT($this->isa('TWiki::Store')) if DEBUG;
     ASSERT($user->isa('TWiki::User')) if DEBUG;
     $web =~ s#\.#/#go;
-	$meta = $this->_removeAutoAttachmentsFromMeta($meta);
-	
+    $meta = $this->_removeAutoAttachmentsFromMeta($meta);
+
     $options = {} unless defined( $options );
 
     if( $user &&
@@ -873,6 +900,7 @@ sub saveTopic {
 =pod
 
 ---++ ObjectMethod saveAttachment ($web, $topic, $attachment, $user, $opts )
+
    * =$user= - user doing the saving
    * =$web= - web for topic
    * =$topic= - topic to atach to
@@ -920,12 +948,11 @@ sub saveAttachment {
         }
 
         if( $opts->{file} && !$opts->{stream} ) {
-            open($opts->{stream}, $opts->{file}) ||
+            open($opts->{stream}, "<$opts->{file}") ||
               throw Error::Simple('Could not open '.$opts->{file} );
             binmode($opts->{stream}) ||
               throw Error::Simple( $opts->{file}.' binmode failed: '.$! );
         }
-
         if ( $opts->{stream} ) {
             $action = 'upload';
 
@@ -965,7 +992,6 @@ sub saveAttachment {
                 open( $opts->{stream}, "<$tmpFile" );
                 binmode( $opts->{stream} );
             }
-
             my $error;
             try {
                 $handler->addRevisionFromStream( $opts->{stream},
@@ -995,6 +1021,7 @@ sub saveAttachment {
 
             $meta->putKeyed( 'FILEATTACHMENT', $attrs );
         } else {
+            $action = 'save';
             $attrs = $meta->get( 'FILEATTACHMENT', $attachment );
             $attrs->{name} = $attachment;
             $attrs->{attr} = ( $opts->{hide} ) ? 'h' : '';
@@ -1013,7 +1040,7 @@ sub saveAttachment {
         $this->unlockTopic( $user, $web, $topic );
     };
 
-    unless( $opts->{dontlog} ) {
+    if ( ( ! $opts->{dontlog} ) && ( $TWiki::cfg{Log}{$action} ) ) {
         $this->{session}->writeLog( $action, $web.'.'.$topic, $attachment, $user );
     }
 }
@@ -1080,11 +1107,13 @@ sub _noHandlersSave {
 =pod
 
 ---++ ObjectMethod repRev( $user, $web, $topic, $text, $meta, $options )
+
 Replace last (top) revision with different text.
 
 Parameters and return value as saveTopic, except
    * =$options= - as for saveTopic, with the extra option:
       * =timetravel= - if we want to force the deposited revision to look as much like the revision specified in =$rev= as possible.
+      * =operation= - set to the name of the operation performing the save. This is used only in the log, and is normally =cmd= or =save=. It defaults to =save=.
 
 Used to try to avoid the deposition of 'unecessary' revisions, for example
 where a user quickly goes back and fixes a spelling error.
@@ -1137,7 +1166,8 @@ sub repRev {
         my $extra = "repRev $rev by " . $revuser->login() .
             ' '. TWiki::Time::formatTime( $revdate, '$rcs', 'gmtime' );
         $extra   .= ' minor' if( $options->{minor} );
-        $this->{session}->writeLog( 'cmd', $web.'.'.$topic, $extra, $user );
+        $this->{session}->writeLog( $options->{operation} || 'save',
+                                    $web.'.'.$topic, $extra, $user );
     }
 }
 
@@ -1236,6 +1266,7 @@ sub lockTopic {
 =pod
 
 ---++ ObjectMethod unlockTopic( $user, $web, $topic )
+
 Release the topic lock on the given topic. A topic lock will cause other
 processes that also try to claim a lock to block. It is important to
 release a topic lock after a guard section is complete. This should
@@ -1374,42 +1405,6 @@ sub extractMetaData {
     return $meta;
 }
 
-sub _extractMetaDataAutoAttachments {
-    my( $this, $user, $web, $topic, $version, $meta ) = @_;
-	if ($TWiki::cfg{AutoAttachPubFiles}) {
-
-#		$this->{session}->writeDebug("$web.$topic META: ".Dumper($meta->{'FILEATTACHMENT'})."\n".Dumper($this));  
-
-# TODO need to ensure that this is not called for every topic examined, but rather just the one we are specifically interested in
-#		return unless ("$web.$topic" eq $this->{session}{webName}.".".$this->{session}{topicName});
-
-	    my @knownAttachments = $meta->find('FILEATTACHMENT');
-	    my $ka = undef; #ugly I know
-	    if ($#knownAttachments) {
-	        $ka = \@knownAttachments;
-		} 
-
-
-
-#		use Data::Dumper;
-#		$Data::Dumper::Maxdepth = 2;
-#		$this->{session}->writeDebug("$web.$topic META =".Dumper($meta));
-#      	$this->{session}->writeDebug("AUTOATTACHING on $web.$topic\nFOUND BEFORE ".Dumper($ka)."\n");
-
-#       $this->{session}->writeDebug("KA2: ".Dumper(@knownAttachments));
-       my @attachmentsFoundInPub =
-         _findAttachments($this, $web, $topic, $ka);
-#       	$this->{session}->writeDebug("FOUND AFTER ".Dumper(\@attachmentsFoundInPub));
-
-		if ($#attachmentsFoundInPub) {
-			$meta->putAll('FILEATTACHMENT', @attachmentsFoundInPub);
-		};
-    } else {
-#       	$this->{session}->writeDebug("NOT AUTOATTACHING on $web.$topic\n ".Dumper($meta)."\n");
-    }
-    return $meta;
-}
-
 =pod
 
 ---++ ObjectMethod getTopicParent (  $web, $topic  ) -> $string
@@ -1712,6 +1707,7 @@ sub createWeb {
 =pod
 
 ---++ ObjectMethod removeWeb( $user, $web )
+
    * =$user= - user doing the removing (for the history)
    * =$web= - web being removed
 
@@ -1872,6 +1868,7 @@ sub cleanUpRevID {
 =pod
 
 ---++ ObjectMethod copyTopic($user, $fromweb, $fromtopic, $toweb, $totopic)
+
 Copy a topic and all it's attendant data from one web to another.
 
 SMELL: Does not fix up meta-data!
@@ -1931,6 +1928,22 @@ sub searchMetaData {
     }
 
     my $text = '';
+    if ($params->{format}) {
+        $text = $this->{session}->{search}->searchWeb
+        (
+         format	       => $params->{format},
+         search        => $searchVal,
+         web           => $attrWeb,
+         type          => 'regex',
+         nosummary     => 'on',
+         nosearch      => 'on',
+         noheader      => 'on',
+         nototal       => 'on',
+         noempty       => 'on',
+         template      => 'searchmeta',
+         inline        => 1,
+        );
+    } else {
     $this->{session}->{search}->searchWeb
       (
        _callback     => \&_collate,
@@ -1946,7 +1959,7 @@ sub searchMetaData {
        template      => 'searchmeta',
        inline        => 1,
       );
-
+    }
     my $attrTitle = $params->{title} || '';
     if( $text ) {
         $text = $attrTitle.$text;
@@ -2002,6 +2015,7 @@ sub searchInWebContent {
 =pod
 
 ---++ ObjectMethod getRevisionAtTime( $web, $topic, $time ) -> $rev
+
    * =$web= - web for topic
    * =$topic= - topic
    * =$time= - time (in epoch secs) for the rev
@@ -2022,6 +2036,7 @@ sub getRevisionAtTime {
 =pod
 
 ---++ ObjectMethod getLease( $web, $topic ) -> $lease
+
    * =$web= - web for topic
    * =$topic= - topic
 
@@ -2090,5 +2105,20 @@ sub clearLease {
     $handler->setLease( undef );
 }
 
+=pod
+
+---++ ObjectMethod removeSpuriousLeases( $web )
+
+Remove leases that are not related to a topic. These can get left behind in
+some store implementations when a topic is created, but never saved.
+
+=cut
+
+sub removeSpuriousLeases {
+    my( $this, $web ) = @_;
+    my $handler = $this->_getHandler( $web );
+    $handler->removeSpuriousLeases();
+}
+
 1;
 
