@@ -1,7 +1,7 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
 # Copyright (C) 2002 John Talintyre, john.talintyre@btinternet.com
-# Copyright (C) 2002-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 2002-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -19,6 +19,7 @@
 # As per the GPL, removal of this notice is prohibited.
 
 =pod
+
 ---+ package TWiki::Store::RcsFile
 
 This class is PACKAGE PRIVATE to Store, and should never be
@@ -47,6 +48,7 @@ use TWiki::Sandbox;
 =pod
 
 ---++ ClassMethod new($session, $web, $topic, $attachment)
+
 Constructor. There is one object per stored file.
 
 Note that $web, $topic and $attachment must be untainted!
@@ -81,11 +83,23 @@ sub new {
             $this->{rcsFile} = $TWiki::cfg{DataDir}.'/'.
               $web.$rcsSubDir.'/'.$topic.'.txt,v';
         }
+
+        # remove utf8 encodings from filenames
+        downgrade($this->{attachment});
+        downgrade($this->{file});
+        downgrade($this->{rcsFile});
     }
 
     return $this;
 }
 
+# downgrade utf8 encoding of files
+sub downgrade {
+  return unless $_[0];
+  return unless $] >= 5.008;
+  utf8::downgrade($_[0]);
+}
+
 # Used in subclasses for late initialisation during object creation
 # (after the object is blessed into the subclass)
 sub init {
@@ -133,7 +147,7 @@ sub _epochToRcsDateTime {
 sub _controlFileName {
     my( $this, $type ) = @_;
 
-    my $fn = $this->{file};
+    my $fn = $this->{file} || '';
     $fn =~ s/txt$/$type/;
     return $fn;
 }
@@ -141,6 +155,7 @@ sub _controlFileName {
 =pod
 
 ---++ ObjectMethod getRevisionInfo($version) -> ($rev, $date, $user, $comment)
+
    * =$version= if 0 or undef, or out of range (version number > number of revs) will return info about the latest revision.
 
 Returns (rev, date, user, comment) where rev is the number of the rev for which the info was recovered, date is the date of that rev (epoch s), user is the login name of the user who saved that rev, and comment is the comment associated with the rev.
@@ -160,6 +175,7 @@ sub getRevisionInfo {
 =pod
 
 ---++ ObjectMethod getLatestRevision() -> $text
+
 Get the text of the most recent revision
 
 =cut
@@ -172,6 +188,7 @@ sub getLatestRevision {
 =pod
 
 ---++ ObjectMethod getLatestRevisionTime() -> $text
+
 Get the time of the most recent revision
 
 =cut
@@ -184,6 +201,7 @@ sub getLatestRevisionTime {
 =pod
 
 ---++ ObjectMethod readMetaData($name) -> $text
+
 Get a meta-data block for this web
 
 =cut
@@ -220,7 +238,7 @@ sub getWorkArea {
 
     unless( -d $dir ) {
         mkdir( $dir ) || throw Error::Simple(
-            'RCS: failed to create '.$key.'work area: '.$! );
+            'RCS: failed to create '.$key.' work area. Check that {RCS}{WorkAreaDir} points to a valid directory that I can read and write to');
     }
     return $dir;
 }
@@ -270,6 +288,7 @@ sub getTopicNames {
 =pod
 
 ---++ ObjectMethod getWebNames() -> @webs
+
 Gets a list of names of subwebs in the current web
 
 =cut
@@ -320,49 +339,24 @@ match per topic, and will not return matching lines).
 sub searchInWebContent {
     my( $this, $searchString, $topics, $options ) = @_;
     ASSERT(defined $options) if DEBUG;
-    my $type = $options->{type} || '';
-
-    # I18N: 'grep' must use locales if needed,
-    # for case-insensitive searching.  See TWiki::setupLocale.
-    my $program = '';
-    # FIXME: For Cygwin grep, do something about -E and -F switches
-    # - best to strip off any switches after first space in
-    # EgrepCmd etc and apply those as argument 1.
-    if( $type eq 'regex' ) {
-        $program = $TWiki::cfg{RCS}{EgrepCmd};
-    } else {
-        $program = $TWiki::cfg{RCS}{FgrepCmd};
-    }
-
-    $program =~ s/%CS{(.*?)\|(.*?)}%/$options->{casesensitive}?$1:$2/ge;
-    $program =~ s/%DET{(.*?)\|(.*?)}%/$options->{files_without_match}?$2:$1/ge;
-
     my $sDir = $TWiki::cfg{DataDir}.'/'.$this->{web}.'/';
-    my $seen = {};
-    # process topics in sets, fix for Codev.ArgumentListIsTooLongForSearch
-    my $maxTopicsInSet = 512; # max number of topics for a grep call
-    my @take = @$topics;
-    my @set = splice( @take, 0, $maxTopicsInSet );
-    my $sandbox = $this->{session}->{sandbox};
-    while( @set ) {
-        @set = map { "$sDir/$_.txt" } @set;
-        my ($matches, $exit ) = $sandbox->sysCommand(
-            $program,
-            TOKEN => $searchString,
-            FILES => \@set);
-        foreach my $match ( split( /\r?\n/, $matches )) {
-            if( $match =~ m/([^\/]*)\.txt(:(.*))?$/ ) {
-                push( @{$seen->{$1}}, $3 );
-            }
-        }
-        @set = splice( @take, 0, $maxTopicsInSet );
+
+    unless ($this->{searchFn}) {
+        eval "require $TWiki::cfg{RCS}{SearchAlgorithm}";
+        die "Bad {RCS}{SearchAlgorithm}; suggest you run configure and select a different algorithm\n$@" if $@;
+        $this->{searchFn} = $TWiki::cfg{RCS}{SearchAlgorithm}.'::search';
     }
-    return $seen;
+
+    no strict 'refs';
+    return &{$this->{searchFn}}($searchString, $topics, $options,
+               $sDir, $this->{session}->{sandbox});
+    use strict 'refs';
 }
 
 =pod
 
 ---++ ObjectMethod moveWeb(  $newWeb )
+
 Move a web.
 
 =cut
@@ -380,6 +374,7 @@ sub moveWeb {
 =pod
 
 ---++ ObjectMethod getRevision($version) -> $text
+
    * =$version= if 0 or undef, or out of range (version number > number of revs) will return the latest revision.
 
 Get the text of the given revision.
@@ -397,6 +392,7 @@ sub getRevision {
 =pod
 
 ---++ ObjectMethod storedDataExists() -> $boolean
+
 Establishes if there is stored data associated with this handler.
 
 =cut
@@ -450,6 +446,7 @@ sub restoreLatestRevision {
 =pod
 
 ---++ ObjectMethod removeWeb( $web )
+
    * =$web= - web being removed
 
 Destroy a web, utterly. Removed the data and attachments in the web.
@@ -471,6 +468,7 @@ sub removeWeb {
 =pod
 
 ---++ ObjectMethod moveTopic( $newWeb, $newTopic )
+
 Move/rename a topic.
 
 =cut
@@ -503,6 +501,7 @@ sub moveTopic {
 =pod
 
 ---++ ObjectMethod copyTopic( $newWeb, $newTopic )
+
 Copy a topic.
 
 =cut
@@ -537,6 +536,7 @@ sub copyTopic {
 =pod
 
 ---++ ObjectMethod moveAttachment( $newWeb, $newTopic, $newAttachment )
+
 Move an attachment from one topic to another. The name is retained.
 
 =cut
@@ -558,6 +558,7 @@ sub moveAttachment {
 =pod
 
 ---++ ObjectMethod copyAttachment( $newWeb, $newTopic )
+
 Copy an attachment from one topic to another. The name is retained.
 
 =cut
@@ -642,6 +643,7 @@ sub isLocked {
 =pod
 
 ---++ ObjectMethod setLease( $lease )
+
    * =$lease= reference to lease hash, or undef if the existing lease is to be cleared.
 
 Set an lease on the topic.
@@ -680,6 +682,30 @@ sub getLease {
     return undef;
 }
 
+=pod
+
+---++ ObjectMethod removeSpuriousLeases( $web )
+
+Remove leases that are not related to a topic. These can get left behind in
+some store implementations when a topic is created, but never saved.
+
+=cut
+
+sub removeSpuriousLeases {
+    my( $this ) = @_;
+    my $web = $TWiki::cfg{DataDir}.'/'.$this->{web}.'/';
+    if (opendir(W, $web)) {
+        foreach my $f (readdir(W)) {
+            if ($f =~ /^(.*)\.lease$/) {
+                if (! -e "$1.txt,v") {
+                    unlink($f);
+                }
+            }
+        }
+        closedir(W);
+    }
+}
+
 sub _saveStream {
     my( $this, $fh ) = @_;
 
@@ -807,7 +833,8 @@ sub _rmtree {
 
     if( opendir( D, $root ) ) {
         foreach my $entry ( grep { !/^\.+$/ } readdir( D ) ) {
-            $entry = $root.'/'.$entry;
+            $entry =~ /^(.*)$/;
+            $entry = $root.'/'.$1;
             if( -d $entry ) {
                 _rmtree( $entry );
             } else {
@@ -908,6 +935,7 @@ Add new revision. Replace file with contents of stream.
 =pod
 
 ---++ ObjectMethod replaceRevision($text, $comment, $user, $date)
+
 Replace the top revision.
    * =$text= is the new revision
    * =$date= is in epoch seconds.
@@ -931,6 +959,7 @@ Delete the last revision - do nothing if there is only one revision
 =pod
 
 ---++ ObjectMethod revisionDiff (   $rev1, $rev2, $contextLines  ) -> \@diffArray
+
 rev2 newer than rev1.
 Return reference to an array of [ diffType, $right, $left ]
 
@@ -1044,6 +1073,7 @@ sub dirForTopicAttachments {
 =pod
 
 ---++ ObjectMethod stringify()
+
 Generate string representation for debugging
 
 =cut
