@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 1999-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 1999-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -40,6 +40,7 @@ use TWiki::UI::Register;
 =pod
 
 ---++ StaticMethod manage( $session )
+
 =manage= command handler.
 This method is designed to be
 invoked via the =TWiki::UI::run= method.
@@ -144,19 +145,13 @@ sub _createWeb {
     my $siteMapUseTo = $query->param( 'sitemapuseto' ) || '';
     my $noSearchAll = $query->param( 'nosearchall' ) || '';
 
-    # check permission, user authorized to create webs?
-#    TWiki::UI::checkAccess( $session, $webName, $topicName,
-#                            'MANAGE', $session->{user} );
-    #hack due to late realisation of hacking possibility
-    unless($session->{user}->isAdmin()) { 
-        throw TWiki::OopsException( 'accessdenied',
-                                    def => 'topic_access',
-                                    web => $webName,
-                                    topic => $topicName,
-                                    params =>
-                                    [ 'MANAGE',
-                                      $session->{i18n}->maketext('access not allowed on web')]);
+    # check permission, user authorized to create web here?
+    my $parent = undef; # default is root if no parent web
+    if( $webName =~ m|^(.*)[./](.*?)$| ) {
+        $parent = $1;
     }
+    TWiki::UI::checkAccess( $session, $parent, undef,
+                            'CHANGE', $session->{user} );
 
     my $newWeb = $query->param( 'newweb' ) || '';
     unless( $newWeb ) {
@@ -218,6 +213,7 @@ sub _createWeb {
 =pod
 
 ---++ StaticMethod rename( $session )
+
 =rename= command handler.
 This method is designed to be
 invoked via the =TWiki::UI::run= method.
@@ -271,11 +267,19 @@ sub rename {
 
     $newTopic =~ s/\s//go;
     $newTopic =~ s/$TWiki::cfg{NameFilter}//go;
+    $newTopic = ucfirst $newTopic;   # Item3270
 
     $attachment ||= '';
 
     TWiki::UI::checkWebExists( $session, $oldWeb, $oldTopic, 'rename' );
-    TWiki::UI::checkTopicExists( $session, $oldWeb, $oldTopic, 'rename');
+    # Item3270: Wrap topic existence into extra try/catch block to
+    #           check for the same name starting with a lower case letter.
+    try {
+        TWiki::UI::checkTopicExists( $session, $oldWeb, $oldTopic, 'rename');
+    } catch TWiki::OopsException with {
+        $oldTopic = lcfirst $oldTopic;
+        TWiki::UI::checkTopicExists( $session, $oldWeb, $oldTopic, 'rename');
+    };
 
     if( $newTopic && !TWiki::isValidWikiWord( $newTopic ) ) {
         unless( $doAllowNonWikiWord ) {
@@ -811,7 +815,8 @@ sub _newTopicScreen {
     $nonWikiWordFlag = 'checked="checked"' if( $doAllowNonWikiWord );
 
     if( $attachment ) {
-        $tmpl = $session->{templates}->readTemplate( $tmplname || 'moveattachment', $skin );
+        $tmpl = $session->{templates}->readTemplate(
+            $tmplname || 'moveattachment', $skin );
         $tmpl =~ s/%FILENAME%/$attachment/go;
     } elsif( $confirm ) {
         $tmpl = $session->{templates}->readTemplate( 'renameconfirm', $skin );
@@ -822,8 +827,8 @@ sub _newTopicScreen {
         $tmpl = $session->{templates}->readTemplate( 'rename', $skin );
     }
 
-    # Trashing a topic; look for a non-conflicting name
-    if( $newWeb eq $TWiki::cfg{TrashWebName} ) {
+    if( !$attachment && $newWeb eq $TWiki::cfg{TrashWebName} ) {
+        # Trashing a topic; look for a non-conflicting name
         $newTopic = $oldWeb.$newTopic;
         my $n = 1;
         my $base = $newTopic;
@@ -837,57 +842,59 @@ sub _newTopicScreen {
     $tmpl =~ s/%NEW_TOPIC%/$newTopic/go;
     $tmpl =~ s/%NONWIKIWORDFLAG%/$nonWikiWordFlag/go;
 
-    my $refs;
-    my %attributes;
-    my %labels;
-    my @keys;
-    my $search = '';
-    if( $currentWebOnly ) {
-        $search = $session->{i18n}->maketext('(skipped)');
-    } else {
-        $refs = getReferringTopics( $session, $oldWeb, $oldTopic, 1 );
-        @keys = sort keys %$refs;
-        foreach my $entry ( @keys ) {
+    if( !$attachment ) {
+        my $refs;
+        my $search = '';
+        if( $currentWebOnly ) {
+            $search = $session->{i18n}->maketext('(skipped)');
+        } else {
+            $refs = getReferringTopics( $session, $oldWeb, $oldTopic, 1 );
+            foreach my $entry ( sort keys %$refs ) {
+                $search .= CGI::Tr
+                  (CGI::td(
+                      { class => 'twikiTopRow' },
+                      CGI::input(
+                          { type => 'checkbox',
+                            class => 'twikiCheckBox',
+                            name => 'referring_topics',
+                            value => $entry,
+                            checked => 'checked' } ). " [[$entry]] " ) .
+                              CGI::td(
+                                  { class => 'twikiSummary twikiGrayText' },
+                                  $refs->{$entry} ));
+            }
+            unless( $search ) {
+                $search = ($session->{i18n}->maketext('(none)'));
+            } else {
+                $search = CGI::start_table().$search.CGI::end_table();
+            }
+        }
+        $tmpl =~ s/%GLOBAL_SEARCH%/$search/o;
+
+        $refs = getReferringTopics( $session, $oldWeb, $oldTopic, 0 );
+
+        $search = '';;
+        foreach my $entry ( sort keys %$refs ) {
             $search .= CGI::Tr
-              (CGI::td
-               ( { class => 'twikiTopRow' },
-                 CGI::input( { type => 'checkbox',
-                               class => 'twikiCheckBox',
-                               name => 'referring_topics',
-                               value => $entry,
-                               checked => 'checked' } ). " [[$entry]] " ) .
-               CGI::td( { class => 'twikiSummary twikiGrayText' },
-                        $refs->{$entry} ));
+              (CGI::td(
+                  { class => 'twikiTopRow' },
+                  CGI::input(
+                      { type => 'checkbox',
+                        class => 'twikiCheckBox',
+                        name => 'referring_topics',
+                        value => $entry,
+                        checked => 'checked' } ). " [[$entry]] " ) .
+                          CGI::td(
+                              { class => 'twikiSummary twikiGrayText' },
+                              $refs->{$entry} ));
         }
         unless( $search ) {
             $search = ($session->{i18n}->maketext('(none)'));
         } else {
             $search = CGI::start_table().$search.CGI::end_table();
         }
+        $tmpl =~ s/%LOCAL_SEARCH%/$search/go;
     }
-    $tmpl =~ s/%GLOBAL_SEARCH%/$search/o;
-
-    $refs = getReferringTopics( $session, $oldWeb, $oldTopic, 0 );
-    @keys = sort keys %$refs;
-    $search = '';;
-    foreach my $entry ( @keys ) {
-        $search .= CGI::Tr
-          (CGI::td
-           ( { class => 'twikiTopRow' },
-             CGI::input( { type => 'checkbox',
-                           class => 'twikiCheckBox',
-                           name => 'referring_topics',
-                           value => $entry,
-                           checked => 'checked' } ). " [[$entry]] " ) .
-           CGI::td( { class => 'twikiSummary twikiGrayText' },
-                    $refs->{$entry} ));
-    }
-    unless( $search ) {
-        $search = ($session->{i18n}->maketext('(none)'));
-    } else {
-        $search = CGI::start_table().$search.CGI::end_table();
-    }
-    $tmpl =~ s/%LOCAL_SEARCH%/$search/go;
 
     $tmpl = $session->handleCommonTags( $tmpl, $oldWeb, $oldTopic );
     $tmpl = $session->{renderer}->getRenderedVersion( $tmpl, $oldWeb, $oldTopic );
@@ -1009,14 +1016,10 @@ sub _newWebScreen {
     $tmpl =~ s/%RENAMEWEB_SUBMIT%/\%$submitAction\%/go;
 
     my $refs;
-    my %attributes;
-    my %labels;
-    my @keys;
     my $search = '';
 
     $refs = ${$webTopicInfoRef}{referring}{refs1};
-    @keys = sort keys %$refs;
-    foreach my $entry ( @keys ) {
+    foreach my $entry ( sort keys %$refs ) {
         $search .= CGI::Tr(
             CGI::td(
                 { class => 'twikiTopRow' },
@@ -1026,10 +1029,11 @@ sub _newWebScreen {
                       name => 'referring_topics',
                       value => $entry,
                       checked => 'checked' } ). " [[$entry]] " ) .
-                 CGI::td( { class => 'twikiSummary twikiGrayText' },
-                          $refs->{$entry}
-                         )
-                  );
+                        CGI::td(
+                            { class => 'twikiSummary twikiGrayText' },
+                            $refs->{$entry}
+                           )
+                         );
     }
     unless( $search ) {
         $search = ($session->{i18n}->maketext('(none)'));
@@ -1039,19 +1043,20 @@ sub _newWebScreen {
     $tmpl =~ s/%GLOBAL_SEARCH%/$search/o;
 
     $refs = $webTopicInfoRef->{referring}{refs0};
-    @keys = sort keys %$refs;
     $search = '';
-    foreach my $entry ( @keys ) {
-        $search .= CGI::Tr
-        (CGI::td
-          ( { class => 'twikiTopRow' },
-            CGI::input( { type => 'checkbox',
-                          class => 'twikiCheckBox',
-                          name => 'referring_topics',
-                          value => $entry,
-                          checked => 'checked' } ). " [[$entry]] " ) .
-                        CGI::td( { class => 'twikiSummary twikiGrayText' },
-                                 $refs->{$entry} ));
+    foreach my $entry ( sort keys %$refs ) {
+        $search .= CGI::Tr(
+            CGI::td(
+                { class => 'twikiTopRow' },
+                CGI::input(
+                    { type => 'checkbox',
+                      class => 'twikiCheckBox',
+                      name => 'referring_topics',
+                      value => $entry,
+                      checked => 'checked' } ). " [[$entry]] " ) .
+                        CGI::td(
+                            { class => 'twikiSummary twikiGrayText' },
+                            $refs->{$entry} ));
     }
     unless( $search ) {
         $search = ($session->{i18n}->maketext('(none)'));
@@ -1081,6 +1086,7 @@ sub _getReferringTopicsListFromURL {
 =pod
 
 ---++ StaticMethod getReferringTopics($session, $web, $topic, $allWebs) -> \%matches
+
    * =$session= - the session
    * =$web= - web to search for
    * =$topic= - topic to search for
