@@ -1,6 +1,6 @@
 # Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
 #
-# Copyright (C) 1999-2006 Peter Thoeny, peter@thoeny.org
+# Copyright (C) 1999-2007 Peter Thoeny, peter@thoeny.org
 # and TWiki Contributors. All Rights Reserved. TWiki Contributors
 # are listed in the AUTHORS file in the root of this distribution.
 # NOTE: Please extend that file, not this notice.
@@ -126,7 +126,7 @@ sub buildNewTopic {
           $session->expandVariablesOnTopicCreation( $templateText );
         foreach my $k ( keys %$templateMeta ) {
             unless( $k =~ /^_/ || $k eq 'FORM' || $k eq 'TOPICPARENT' ||
-                      $k eq 'FIELD' ) {
+                      $k eq 'FIELD' || $k eq 'TOPICMOVED' ) {
                 $newMeta->copyFrom( $templateMeta, $k );
             }
         }
@@ -274,6 +274,25 @@ sub buildNewTopic {
     return( $newMeta, $newText, $saveOpts, $merged );
 }
 
+sub _getRedirectUrl {
+    my $session = shift;
+
+    my $query = $session->{cgiQuery};
+    my $redirecturl = $query->param( 'redirectto' );
+    return '' unless $redirecturl;
+    if( $redirecturl =~ /^$TWiki::regex{linkProtocolPattern}\:\/\//o ) {
+        # assuming URL
+        if ($TWiki::cfg{AllowRedirectUrl}) {
+            return $redirecturl;
+        } else {
+            return '';
+        }
+    }
+    # assuming 'web.topic' or 'topic'
+    my ( $w, $t ) = $session->normalizeWebTopicName( $session->{webName}, $redirecturl );
+    $redirecturl = $session->getScriptUrl( 1, 'view', $w, $t );
+}
+
 =pod
 
 ---++ StaticMethod save($session)
@@ -298,13 +317,10 @@ sub save {
     my $store = $session->{store};
     my $user = $session->{user};
 
-    $session->enterContext( 'save' );
-
-    #
-    # Allow for dynamic topic creation by replacing strings of at least
-    # 10 x's XXXXXX with a next-in-sequence number.
-    # http://twiki.org/cgi-bin/view/Codev/AllowDynamicTopicNameCreation
-    #
+    # Do not remove, keep as undocumented feature for compatibility with
+    # TWiki 4.0.x: Allow for dynamic topic creation by replacing strings
+    # of at least 10 x's XXXXXX with a next-in-sequence number.
+    # See Codev.AllowDynamicTopicNameCreation
     if ( $topic =~ /X{10}/ ) {
 		my $n = 0;
 		my $baseTopic = $topic;
@@ -317,8 +333,38 @@ sub save {
         $session->{topicName} = $topic;
     }
 
-    my $redirecturl = $session->getScriptUrl( 1, 'view', $web, $topic );
+    # Allow for more flexible topic creation with sortable names and
+    # better performance. See Codev.AutoIncTopicNameOnSave
+    if( $topic =~ /AUTOINC([0-9]+)/ ) {
+        my $start = $1;
+        my $baseTopic = $topic;
+        $store->clearLease( $web, $baseTopic );
+        my $nameFilter = $topic;
+        $nameFilter =~ s/AUTOINC([0-9]+)/([0-9]+)/;
+        my @list =
+          sort{ $a <=> $b }
+            map{ s/^$nameFilter$/$1/; s/^0*([0-9])/$1/; $_ }
+              grep{ /^$nameFilter$/ }
+                $store->getTopicNames( $web );
+        if( scalar @list ) {
+            # find last one, and increment by one
+            my $next = $list[$#list] + 1;
+            my $len = length( $start );
+            $start =~ s/^0*([0-9])/$1/; # cut leading zeros
+            $next = $start if( $start > $next );
+            my $pad =  $len - length($next);
+            if( $pad > 0 ) {
+                $next = '0' x $pad . $next; # zero-pad
+            }
+            $topic =~ s/AUTOINC[0-9]+/$next/;
+        } else {
+            # first auto-inc topic
+            $topic =~ s/AUTOINC[0-9]+/$start/;
+        }
+        $session->{topicName} = $topic;
+    }
 
+    my $redirecturl = _getRedirectUrl( $session );
     my $saveaction = '';
     foreach my $action qw( save checkpoint quietsave cancel preview
                            addform replaceform delRev repRev ) {
@@ -332,7 +378,9 @@ sub save {
     # for compatibility with old templates.
     if( !$saveaction && $query->param( 'action' )) {
         $saveaction = lc($query->param( 'action' ));
-        $session->writeWarning('Use of deprecated "action" parameter to "save". Correct your templates!');
+        $session->writeWarning(<<WARN);
+Use of deprecated "action" parameter to "save". Correct your templates!
+WARN
 
         # handle old values for form-related actions:
         $saveaction = 'addform' if ( $saveaction eq 'add form');
@@ -346,16 +394,18 @@ sub save {
         }
 
         # redirect to a sensible place (a topic that exists)
-        my( $w, $t, $a ) = ( '', '', '?unlock=on' );
-        foreach my $test ( $topic,
-                     $query->param( 'topicparent' ),
-                     $TWiki::cfg{HomeTopicName} ) {
-            ( $w, $t ) =
-              $session->normalizeWebTopicName( $web, $test );
-            last if( $store->topicExists( $w, $t ));
-            $a = '';
+        my $viewURL = $redirecturl || '';
+        unless ( $viewURL ) {
+            my( $w, $t ) = ( '', '' );
+            foreach my $test ( $topic,
+                               $query->param( 'topicparent' ),
+                               $TWiki::cfg{HomeTopicName} ) {
+                ( $w, $t ) =
+                  $session->normalizeWebTopicName( $web, $test );
+                last if( $store->topicExists( $w, $t ));
+            }
+            $viewURL = $session->getScriptUrl( 1, 'view', $w, $t );
         }
-        my $viewURL = $session->getScriptUrl( 1, 'view', $w, $t );
         $session->redirect( $viewURL );
 
         return;
@@ -368,7 +418,10 @@ sub save {
     }
 
     my $editaction = lc($query->param( 'editaction' )) || '';
+    my $edit = $query->param( 'edit' ) || 'edit';
+    my $editparams = $query->param( 'editparams' ) || '';
 
+    ## SMELL: The form affecting actions do not preserve edit and editparams
     if( $saveaction eq 'addform' ||
           $saveaction eq 'replaceform' ||
             $saveaction eq 'preview' && $query->param( 'submitChangeForm' )) {
@@ -381,11 +434,18 @@ sub save {
 
     if( $saveaction eq 'checkpoint' ) {
         $query->param( -name=>'dontnotify', -value=>'checked' );
-        my $editURL = $session->getScriptUrl( 1, 'edit', $web, $topic );
+        my $editURL = $session->getScriptUrl( 1, $edit, $web, $topic );
         $redirecturl = $editURL.'?t='.time();
+        $redirecturl .= '&redirectto='.$query->param( 'redirectto' )
+          if $query->param( 'redirectto' );
+        # select the appropriate edit template
         $redirecturl .= '&action='.$editaction if $editaction;
-        $redirecturl .= '&skin='.$query->param('skin') if $query->param('skin');
-        $redirecturl .= '&cover='.$query->param('cover') if $query->param('cover');
+        $redirecturl .= '&skin='.$query->param('skin')
+          if $query->param('skin');
+        $redirecturl .= '&cover='.$query->param('cover')
+          if $query->param('cover');
+        $redirecturl .= $editparams
+          if $editparams;  # May contain anchor
         my $lease = $store->getLease( $web, $topic );
         if( $lease && $lease->{user}->equals( $user )) {
             $store->setLease( $web, $topic, $user, $TWiki::cfg{LeaseLength} );
@@ -415,6 +475,8 @@ sub save {
                                       '.'.$TWiki::cfg{SuperAdminGroup} );
     }
 
+    $redirecturl ||= $session->getScriptUrl( 1, 'view', $web, $topic );
+
     if( $saveCmd eq 'delRev' ) {
         # delete top revision
         try {
@@ -438,7 +500,10 @@ sub save {
         my $textQueryParam = $query->param( 'text' );
         my $meta = new TWiki::Meta( $session, $web, $topic );
         $store->extractMetaData( $meta, \$textQueryParam );
-        my $saveOpts = { timetravel => 1 };
+        my $saveOpts = {
+            timetravel => 1,
+            operation => 'cmd',
+           };
         try {
             $store->repRev( $user, $web, $topic,
                             $textQueryParam, $meta, $saveOpts );
